# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from ecl.network.v2 import _proxy
from ecl.network.v2 import extension
from ecl.network.v2 import load_balancer
from ecl.network.v2 import network
from ecl.network.v2 import port
from ecl.network.v2 import quota
from ecl.network.v2 import subnet
from ecl.network.v2 import vpn
from ecl.tests.unit import test_proxy_base2


class TestNetworkProxy(test_proxy_base2.TestProxyBase):
    def setUp(self):
        super(TestNetworkProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_load_balancer_create_attrs(self):
        self.verify_create(self.proxy.create_load_balancer,
                           load_balancer.LoadBalancer, 
                           method_kwargs={"load_balancer_plan_id":"1212"})

    def test_load_balancer_delete(self):
        self.verify_delete(self.proxy.delete_load_balancer,
                           load_balancer.LoadBalancer, 
                           False,
                           method_kwargs={})

    def test_load_balancer_delete_ignore(self):
        self.verify_delete(self.proxy.delete_load_balancer,
                           load_balancer.LoadBalancer, True, 
                           method_kwargs={})

    def test_load_balancer_get(self):
        self.verify_get(self.proxy.get_load_balancer,
                        load_balancer.LoadBalancer)

    def test_load_balancers(self):
        self.verify_list(self.proxy.load_balancers,
                         load_balancer.LoadBalancer,
                         paginated=False)

    def test_load_balancer_update(self):
        self.verify_update(self.proxy.update_load_balancer,
                           load_balancer.LoadBalancer)

    def test_network_create_attrs(self):
        self.verify_create(self.proxy.create_network, network.Network, 
                           method_kwargs={"name":"test"})

    def test_network_delete(self):
        self.verify_delete(self.proxy.delete_network, network.Network, False)

    def test_network_delete_ignore(self):
        self.verify_delete(self.proxy.delete_network, network.Network, True)

    def test_network_find(self):
        self.verify_find(self.proxy.find_network, network.Network)

    def test_network_get(self):
        self.verify_get(self.proxy.get_network, network.Network)

    def test_networks(self):
        self.verify_list(self.proxy.networks, network.Network,
                         paginated=False)

    def test_network_update(self):
        self.verify_update(self.proxy.update_network, network.Network)

    def test_port_create_attrs(self):
        self.verify_create(self.proxy.create_port, port.Port, 
                           method_kwargs={"name":"test"})

    def test_port_delete(self):
        self.verify_delete(self.proxy.delete_port, port.Port, False)

    def test_port_delete_ignore(self):
        self.verify_delete(self.proxy.delete_port, port.Port, True)

    def test_port_find(self):
        self.verify_find(self.proxy.find_port, port.Port)

    def test_port_get(self):
        self.verify_get(self.proxy.get_port, port.Port)

    def test_ports(self):
        self.verify_list(self.proxy.ports, port.Port, paginated=False)

    def test_port_update(self):
        self.verify_update(self.proxy.update_port, port.Port)

    def test_quota_get(self):
        self.verify_get(self.proxy.get_quota, quota.Quota)

    # def test_quota_default_get(self):
    #     self.verify_get(self.proxy.get_quota_default, quota.QuotaDefault,
    #                     value=[mock.sentinel.project_id],
    #                     expected_args=[quota.QuotaDefault],
    #                     expected_kwargs={"path_args": {
    #                         "project": mock.sentinel.project_id}})

    def test_subnet_create_attrs(self):
        self.verify_create(self.proxy.create_subnet, subnet.Subnet, 
                           method_kwargs={"network_id": "test_id", 
                                          "cidr": "12121"})

    def test_subnet_delete(self):
        self.verify_delete(self.proxy.delete_subnet, subnet.Subnet, False)

    def test_subnet_delete_ignore(self):
        self.verify_delete(self.proxy.delete_subnet, subnet.Subnet, True)

    def test_subnet_find(self):
        self.verify_find(self.proxy.find_subnet, subnet.Subnet)

    def test_subnet_get(self):
        self.verify_get(self.proxy.get_subnet, subnet.Subnet)

    def test_subnets(self):
        self.verify_list(self.proxy.subnets, subnet.Subnet, paginated=False)

    def test_subnet_update(self):
        self.verify_update(self.proxy.update_subnet, subnet.Subnet)

    def test_vpn_service_get(self):
        self.verify_get(self.proxy.get_vpn_service, vpn.VPNService)

    def test_vpn_services(self):
        self.verify_list(self.proxy.vpn_services, vpn.VPNService,
                         paginated=False, 
                         mock_method="ecl.proxy2.BaseProxy._list")




# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import six
import testtools

from ecl import exceptions
from ecl.orchestration.v1 import stack
from ecl import resource2 as resource


FAKE_ID = 'ce8ae86c-9810-4cb1-8888-7fb53bc523bf'
FAKE_NAME = 'test_stack'
FAKE = {
    'capabilities': '1',
    'creation_time': '2015-03-09T12:15:57.233772',
    'description': '3',
    'disable_rollback': True,
    'id': FAKE_ID,
    'links': [{
        'href': 'stacks/%s/%s' % (FAKE_NAME, FAKE_ID),
        'rel': 'self'}],
    'notification_topics': '7',
    'outputs': '8',
    'parameters': {'OS::stack_id': '9'},
    'name': FAKE_NAME,
    'status': '11',
    'status_reason': '12',
    'template_description': '13',
    'template_url': 'http://www.example.com/wordpress.yaml',
    'timeout_mins': '14',
    'updated_time': '2015-03-09T12:30:00.000000',
    'environment': {'network_id': 'ad936ae4-2983-4f23-9187-e47e82cb2725'},
}
FAKE_CREATE_RESPONSE = {
    'stack': {
        'id': FAKE_ID,
        'links': [{
            'href': 'stacks/%s/%s' % (FAKE_NAME, FAKE_ID),
            'rel': 'self'}]}
}


class TestStack(testtools.TestCase):

    def test_basic(self):
        sot = stack.Stack()
        self.assertEqual('stack', sot.resource_key)
        self.assertEqual('stacks', sot.resources_key)
        self.assertEqual('/stacks', sot.base_path)
        self.assertEqual('orchestration', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = stack.Stack(**FAKE)
        self.assertEqual(FAKE['capabilities'], sot.capabilities)
        self.assertEqual(FAKE['creation_time'], sot.created_at)
        self.assertEqual(FAKE['description'], sot.description)
        self.assertTrue(sot.is_rollback_disabled)
        self.assertEqual(FAKE['id'], sot.id)
        self.assertEqual(FAKE['links'], sot.links)
        self.assertEqual(FAKE['notification_topics'],
                         sot.notification_topics)
        self.assertEqual(FAKE['outputs'], sot.outputs)
        self.assertEqual(FAKE['parameters'], sot.parameters)
        self.assertEqual(FAKE['name'], sot.name)
        self.assertEqual(FAKE['status'], sot.status)
        self.assertEqual(FAKE['status_reason'],
                         sot.status_reason)
        self.assertEqual(FAKE['template_description'],
                         sot.template_description)
        self.assertEqual(FAKE['template_url'], sot.template_url)
        self.assertEqual(FAKE['timeout_mins'], sot.timeout_mins)
        self.assertEqual(FAKE['updated_time'], sot.updated_at)
        self.assertEqual(FAKE['environment'], sot.environment)

    @mock.patch.object(resource.Resource, 'create')
    def test_create(self, mock_create):
        sess = mock.Mock()
        sot = stack.Stack(FAKE)

        res = sot.create(sess)

        mock_create.assert_called_once_with(sess, prepend_key=False)
        self.assertEqual(mock_create.return_value, res)

    @mock.patch.object(resource.Resource, 'update')
    def test_update(self, mock_update):
        sess = mock.Mock()
        sot = stack.Stack(FAKE)

        res = sot.update(sess)

        mock_update.assert_called_once_with(sess, prepend_key=False,
                                            has_body=False)
        self.assertEqual(mock_update.return_value, res)

    def test_check(self):
        sess = mock.Mock()
        sot = stack.Stack(**FAKE)
        sot._action = mock.Mock()
        body = {'check': ''}

        sot.check(sess)

        sot._action.assert_called_with(sess, body)

    @mock.patch.object(resource.Resource, 'get')
    def test_get(self, mock_get):
        sess = mock.Mock()
        sot = stack.Stack(**FAKE)
        deleted_stack = mock.Mock(id=FAKE_ID, status='DELETE_COMPLETE')
        normal_stack = mock.Mock(status='CREATE_COMPLETE')
        mock_get.side_effect = [
            normal_stack,
            exceptions.NotFoundException(message='oops'),
            deleted_stack,
        ]

        self.assertEqual(normal_stack, sot.get(sess))
        ex = self.assertRaises(exceptions.NotFoundException, sot.get, sess)
        self.assertEqual('NotFoundException: oops', six.text_type(ex))
        ex = self.assertRaises(exceptions.NotFoundException, sot.get, sess)
        self.assertEqual('NotFoundException: No stack found for %s' % FAKE_ID,
                         six.text_type(ex))


class TestStackPreview(testtools.TestCase):

    def test_basic(self):
        sot = stack.StackPreview()

        self.assertEqual('/stacks/preview', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from ecl.compute.v2 import server

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'accessIPv4': '1',
    'accessIPv6': '2',
    'addresses': {'region': '3'},
    'created': '2015-03-09T12:14:57.233772',
    'flavorRef': '5',
    'flavor': {'id': 'FLAVOR_ID', 'links': {}},
    'hostId': '6',
    'id': IDENTIFIER,
    'imageRef': '8',
    'image': {'id': 'IMAGE_ID', 'links': {}},
    'links': '9',
    'metadata': {'key': '10'},
    'name': '11',
    'progress': 12,
    'tenant_id': '13',
    'status': '14',
    'updated': '2015-03-09T12:15:57.233772',
    'user_id': '16',
    'key_name': '17',
    'OS-DCF:diskConfig': '18',
    'OS-EXT-AZ:availability_zone': '19',
    'OS-EXT-STS:power_state': '20',
    'OS-EXT-STS:task_state': '21',
    'OS-EXT-STS:vm_state': '22',
    'os-extended-volumes:volumes_attached': '23',
    'OS-SRV-USG:launched_at': '2015-03-09T12:15:57.233772',
    'OS-SRV-USG:terminated_at': '2015-03-09T12:15:57.233772',
    'security_groups': '26',
    'adminPass': '27',
    'personality': '28',
    'block_device_mapping_v2': {'key': '29'},
    'os:scheduler_hints': {'key': '30'},
    'user_data': '31'
}


class TestServer(testtools.TestCase):

    def setUp(self):
        super(TestServer, self).setUp()
        self.resp = mock.Mock()
        self.resp.body = None
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.sess = mock.Mock()
        self.sess.post = mock.Mock(return_value=self.resp)

    def test_basic(self):
        sot = server.Server()
        self.assertEqual('server', sot.resource_key)
        self.assertEqual('servers', sot.resources_key)
        self.assertEqual('/servers', sot.base_path)
        self.assertEqual('compute', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual({"image": "image",
                              "flavor": "flavor",
                              "name": "name",
                              "status": "status",
                              "host": "host",
                              "changes_since": "changes-since"},
                             sot._query_mapping._mapping)

    def test_make_it(self):
        sot = server.Server(**EXAMPLE)
        self.assertEqual(EXAMPLE['accessIPv4'], sot.access_ipv4)
        self.assertEqual(EXAMPLE['accessIPv6'], sot.access_ipv6)
        self.assertEqual(EXAMPLE['addresses'], sot.addresses)
        self.assertEqual(EXAMPLE['created'], sot.created_at)
        self.assertEqual(EXAMPLE['flavorRef'], sot.flavor_id)
        self.assertEqual(EXAMPLE['flavor'], sot.flavor)
        self.assertEqual(EXAMPLE['hostId'], sot.host_id)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['imageRef'], sot.image_id)
        self.assertEqual(EXAMPLE['image'], sot.image)
        self.assertEqual(EXAMPLE['links'], sot.links)
        self.assertEqual(EXAMPLE['metadata'], sot.metadata)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['progress'], sot.progress)
        self.assertEqual(EXAMPLE['tenant_id'], sot.project_id)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['updated'], sot.updated_at)
        self.assertEqual(EXAMPLE['user_id'], sot.user_id)
        self.assertEqual(EXAMPLE['key_name'], sot.key_name)
        self.assertEqual(EXAMPLE['OS-DCF:diskConfig'], sot.disk_config)
        self.assertEqual(EXAMPLE['OS-EXT-AZ:availability_zone'],
                         sot.availability_zone)
        self.assertEqual(EXAMPLE['OS-EXT-STS:power_state'], sot.power_state)
        self.assertEqual(EXAMPLE['OS-EXT-STS:task_state'], sot.task_state)
        self.assertEqual(EXAMPLE['OS-EXT-STS:vm_state'], sot.vm_state)
        self.assertEqual(EXAMPLE['os-extended-volumes:volumes_attached'],
                         sot.attached_volumes)
        self.assertEqual(EXAMPLE['OS-SRV-USG:launched_at'], sot.launched_at)
        self.assertEqual(EXAMPLE['OS-SRV-USG:terminated_at'],
                         sot.terminated_at)
        self.assertEqual(EXAMPLE['security_groups'], sot.security_groups)
        self.assertEqual(EXAMPLE['adminPass'], sot.admin_pass)
        self.assertEqual(EXAMPLE['adminPass'], sot.adminPass)
        self.assertEqual(EXAMPLE['personality'], sot.personality)
        self.assertEqual(EXAMPLE['block_device_mapping_v2'],
                         sot.block_device_mapping_v2)
        self.assertEqual(EXAMPLE['os:scheduler_hints'], sot.scheduler_hints)
        self.assertEqual(EXAMPLE['user_data'], sot.user_data)

    def test_detail(self):
        sot = server.ServerDetail()
        self.assertEqual('server', sot.resource_key)
        self.assertEqual('servers', sot.resources_key)
        self.assertEqual('/servers/detail', sot.base_path)
        self.assertEqual('compute', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_change_passowrd(self):
        sot = server.Server(**EXAMPLE)

        self.assertIsNone(sot.change_password(self.sess, 'a'))

        url = 'servers/IDENTIFIER/action'
        body = {"changePassword": {"adminPass": "a"}}
        headers = {'Accept': ''}
        self.sess.post.assert_called_with(
            url, endpoint_filter=sot.service, json=body, headers=headers)

    def test_reboot(self):
        sot = server.Server(**EXAMPLE)

        self.assertIsNone(sot.reboot(self.sess, 'HARD'))

        url = 'servers/IDENTIFIER/action'
        body = {"reboot": {"type": "HARD"}}
        headers = {'Accept': ''}
        self.sess.post.assert_called_with(
            url, endpoint_filter=sot.service, json=body, headers=headers)

    def test_force_delete(self):
        sot = server.Server(**EXAMPLE)

        self.assertIsNone(sot.force_delete(self.sess))

        url = 'servers/IDENTIFIER/action'
        body = {'forceDelete': None}
        headers = {'Accept': ''}
        self.sess.post.assert_called_with(
            url, endpoint_filter=sot.service, json=body, headers=headers)

    def test_rebuild(self):
        sot = server.Server(**EXAMPLE)
        # Let the translate pass through, that portion is tested elsewhere
        sot._translate_response = lambda arg: arg

        result = sot.rebuild(self.sess, name='noo', admin_password='seekr3t',
                             image='http://image/1', access_ipv4="12.34.56.78",
                             access_ipv6="fe80::100",
                             metadata={"meta var": "meta val"},
                             personality=[{"path": "/etc/motd",
                                           "contents": "foo"}])

        self.assertIsInstance(result, server.Server)

        url = 'servers/IDENTIFIER/action'
        body = {
            "rebuild": {
                "name": "noo",
                "imageRef": "http://image/1",
                "adminPass": "seekr3t",
                "accessIPv4": "12.34.56.78",
                "accessIPv6": "fe80::100",
                "metadata": {"meta var": "meta val"},
                "personality": [{"path": "/etc/motd", "contents": "foo"}],
                "preserve_ephemeral": False
            }
        }
        headers = {'Accept': ''}
        self.sess.post.assert_called_with(
            url, endpoint_filter=sot.service, json=body, headers=headers)

    def test_rebuild_minimal(self):
        sot = server.Server(**EXAMPLE)
        # Let the translate pass through, that portion is tested elsewhere
        sot._translate_response = lambda arg: arg

        result = sot.rebuild(self.sess, name='nootoo',
                             admin_password='seekr3two',
                             image='http://image/2')

        self.assertIsInstance(result, server.Server)

        url = 'servers/IDENTIFIER/action'
        body = {
            "rebuild": {
                "name": "nootoo",
                "imageRef": "http://image/2",
                "adminPass": "seekr3two",
                "preserve_ephemeral": False
            }
        }
        headers = {'Accept': ''}
        self.sess.post.assert_called_with(
            url, endpoint_filter=sot.service, json=body, headers=headers)

    def test_resize(self):
        sot = server.Server(**EXAMPLE)

        self.assertIsNone(sot.resize(self.sess, '2'))

        url = 'servers/IDENTIFIER/action'
        body = {"resize": {"flavorRef": "2", 
                           "OS-DCF:diskConfig": "AUTO"}}
        headers = {'Accept': ''}
        self.sess.post.assert_called_with(
            url, endpoint_filter=sot.service, json=body, headers=headers)

    def test_confirm_resize(self):
        sot = server.Server(**EXAMPLE)

        self.assertIsNone(sot.confirm_resize(self.sess))

        url = 'servers/IDENTIFIER/action'
        body = {"confirmResize": None}
        headers = {'Accept': ''}
        self.sess.post.assert_called_with(
            url, endpoint_filter=sot.service, json=body, headers=headers)

    def test_revert_resize(self):
        sot = server.Server(**EXAMPLE)

        self.assertIsNone(sot.revert_resize(self.sess))

        url = 'servers/IDENTIFIER/action'
        body = {"revertResize": None}
        headers = {'Accept': ''}
        self.sess.post.assert_called_with(
            url, endpoint_filter=sot.service, json=body, headers=headers)

    def test_create_image(self):
        sot = server.Server(**EXAMPLE)
        name = 'noo'
        metadata = {'nu': 'image', 'created': 'today'}

        self.assertIsNotNone(sot.create_image(self.sess, name, metadata))

        url = 'servers/IDENTIFIER/action'
        body = {"createImage": {'name': name, 'metadata': metadata}}
        headers = {'Accept': ''}
        self.sess.post.assert_called_with(
            url, endpoint_filter=sot.service, json=body, headers=headers)

    def test_create_image_minimal(self):
        sot = server.Server(**EXAMPLE)
        name = 'noo'

        self.assertIsNone(self.resp.body, sot.create_image(self.sess, name))

        url = 'servers/IDENTIFIER/action'
        body = {"createImage": {'name': name}}
        headers = {'Accept': ''}
        self.sess.post.assert_called_with(
            url, endpoint_filter=dict(sot.service), json=body, headers=headers)

    def test_add_security_group(self):
        sot = server.Server(**EXAMPLE)

        self.assertIsNone(sot.add_security_group(self.sess, "group"))

        url = 'servers/IDENTIFIER/action'
        body = {"addSecurityGroup": {"name": "group"}}
        headers = {'Accept': ''}
        self.sess.post.assert_called_with(
            url, endpoint_filter=sot.service, json=body, headers=headers)

    def test_remove_security_group(self):
        sot = server.Server(**EXAMPLE)

        self.assertIsNone(sot.remove_security_group(self.sess, "group"))

        url = 'servers/IDENTIFIER/action'
        body = {"removeSecurityGroup": {"name": "group"}}
        headers = {'Accept': ''}
        self.sess.post.assert_called_with(
            url, endpoint_filter=sot.service, json=body, headers=headers)



# -*- coding: utf-8 -*-

from ecl.provider_connectivity import provider_connectivity_service
from ecl import resource2
from ecl.network.v2 import network
from ecl.network.v2 import subnet
import hashlib


class AddressAssignment(resource2.Resource):
    resources_key = "address_assignments"
    resource_key = "address_assignment"

    service = provider_connectivity_service.ProviderConnectivityService("v2.0")
    base_path = '/' + service.version + \
                '/tenant_connection_requests/' \
                '%(tenant_connection_request_id)s/address_assignments'

    # capabilities
    allow_list = True

    #: tenant_connection_request unique ID.
    tenant_connection_request_id = resource2.URI(
                                            "tenant_connection_request_id")

    #: tenant_connection unique ID.
    tenant_connection_id = resource2.Body("tenant_connection_id")

    #: Network unique ID
    network_id = resource2.Body("network_id")

    #: mac address assigned with port
    mac_address = resource2.Body("mac_address")

    #: List of fixes IP addresses assign to port.
    fixed_ips = resource2.Body("fixed_ips")

    #: Allowed address pairs
    allowed_address_pairs = resource2.Body("allowed_address_pairs")

    @staticmethod
    def _get_id(value):
        if isinstance(value, resource2.Resource):
            # Don't check _alternate_id unless we need to. It's an uncommon
            # case and it involves looping through the class' dict.
            id = value.id or getattr(
                value, value._alternate_id(),
                hashlib.new('md5', str(value)).hexdigest())
            return id
        else:
            return value

    def __getattribute__(self, name):
        """Return an attribute on this instance

        This is mostly a pass-through except for a specialization on
        the 'id' name, as this can exist under a different name via the
        `alternate_id` argument to resource.Body.
        """
        if name == "id":
            if name in self._body:
                return self._body[name]
            elif self._alternate_id():
                return self._body[self._alternate_id()]
            else:
                return hashlib.new('md5', str(self)).hexdigest()
        else:
            return object.__getattribute__(self, name)


class ICCNetwork(network.Network):

    service = provider_connectivity_service.ProviderConnectivityService("v2.0")
    base_path = '/' + service.version + \
                '/tenant_connection_requests/' \
                '%(tenant_connection_request_id)s/network'

    # Capabilities
    allow_list = False
    allow_create = False
    allow_delete = False
    allow_update = False
    allow_get = True

    def get(self, session, tenant_connection_request_id):
        uri = self.base_path % {
            "tenant_connection_request_id": tenant_connection_request_id
        }
        resp = session.get(uri, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self


class ICCSubnet(subnet.Subnet):

    service = provider_connectivity_service.ProviderConnectivityService("v2.0")
    base_path = '/' + service.version + \
                '/tenant_connection_requests/' \
                '%(tenant_connection_request_id)s/subnets'

    id = resource2.Body("id")
    tenant_connection_request_id = resource2.URI(
                                            "tenant_connection_request_id")

    # Capabilities
    allow_list = True
    allow_create = False
    allow_delete = False
    allow_update = False
    allow_get = True

    dhcp_server_address = resource2.Body('dhcp_server_address')




# -*- coding: utf-8 -*-

from ecl import resource2


class ProviderConnectivityBaseResource(resource2.Resource):

    @property
    def name_or_id(self):
        try:
            return (self._body.get('name') or
                    '(%s)' % self._body['id'][:13])
        except KeyError:
            pass

    @property
    def name_other_or_id(self):
        try:
            return (self._body.get('name_other') or
                    '(%s)' % self._body['id'][:13])
        except KeyError:
            pass




# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.network.v2 import load_balancer

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    "admin_username": "admin",
    "admin_password": "pass",
    "availability_zone": "zoneA",
    "default_gateway": None,
    "description": "test-descr",
    "id": IDENTIFIER,
    "interfaces": [
      {
        "id": IDENTIFIER,
        "ip_address": "100.100.100.100",
        "name": "Interface 1/1",
        "network_id": IDENTIFIER,
        "slot_number": 1,
        "status": "ACTIVE",
        "type": "user",
        "virtual_ip_address": None,
        "virtual_ip_properties": None
      },
      {
        "id": IDENTIFIER,
        "ip_address": "100.100.100.100",
        "name": "Interface 1/1",
        "network_id": IDENTIFIER,
        "slot_number": 2,
        "status": "ACTIVE",
        "type": "user",
        "virtual_ip_address": None,
        "virtual_ip_properties": None
      },
      {
        "id": IDENTIFIER,
        "ip_address": "100.100.100.100",
        "name": "Interface 1/1",
        "network_id": IDENTIFIER,
        "slot_number": 3,
        "status": "ACTIVE",
        "type": "user",
        "virtual_ip_address": None,
        "virtual_ip_properties": None
      },
      {
        "id": IDENTIFIER,
        "ip_address": "100.100.100.100",
        "name": "Interface 1/1",
        "network_id": IDENTIFIER,
        "slot_number": 4,
        "status": "ACTIVE",
        "type": "user",
        "virtual_ip_address": None,
        "virtual_ip_properties": None
      },
    ],
    "load_balancer_plan_id": IDENTIFIER,
    "name": "test-name",
    "status": "ACTIVE",
    "tenant_id": IDENTIFIER,
    "user_username": "user",
    "user_password": "pass",
  }


class TestLoadBalancer(testtools.TestCase):

    def test_basic(self):
        sot = load_balancer.LoadBalancer()
        self.assertEqual('load_balancer', sot.resource_key)
        self.assertEqual('load_balancers', sot.resources_key)
        self.assertEqual('/load_balancers', sot.base_path)
        self.assertEqual('network', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = load_balancer.LoadBalancer(**EXAMPLE)
        self.assertEqual(EXAMPLE['admin_username'], sot.admin_username)
        self.assertEqual(EXAMPLE['availability_zone'], sot.availability_zone)
        self.assertEqual(EXAMPLE['default_gateway'], sot.default_gateway)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['interfaces'], sot.interfaces)
        self.assertEqual(EXAMPLE['tenant_id'], sot.tenant_id)
        self.assertEqual(EXAMPLE['load_balancer_plan_id'], 
                         sot.load_balancer_plan_id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['user_username'], sot.user_username)



# -*- coding: utf-8 -*-

from ecl.provider_connectivity.v1 import aws_connection as _aws_connection
from ecl.provider_connectivity.v1 import exchange_point as _exchange_point
from ecl.provider_connectivity.v1 import operation as _operation
from ecl.provider_connectivity.v1 import \
            tenant_connection_request as _tc_request
from ecl.provider_connectivity.v1 import \
            tenant_connection as _tenant_connection
from ecl.provider_connectivity.v1 import \
            address_assignment as _addr_assignment
from ecl import proxy2


class Proxy(proxy2.BaseProxy):

    def aws_connections(self, **params):
        """List connection resource between ECL2.0 and AWS.

        :param params: The parameters as query string
            to get connections by specified condition.
        :returns: A list of connection objects
        :rtype: list of :class:
                `~ecl.provider_connectivity.v1.aws_connection.AWSConnection`
        """
        return list(self._list(_aws_connection.AWSConnection,
                               paginated=False,
                               **params))

    def get_aws_connection(self, connection_id):
        """Show connection resource between ECL2.0 and AWS.

        :param string connection_id: ID of specified connection.
        :return: :class:`
                ~ecl.provider_connectivity.v1.aws_connection.AWSConnection`
        """
        return self._get(_aws_connection.AWSConnection, connection_id)

    def find_aws_connection(self, name_or_id, ignore_missing=False):
        """Find a single connection

        :param name_or_id: The name or ID of a connection.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:
                `~ecl.network.v2.connection.AWSConnection` or None
        """
        return self._find(_aws_connection.AWSConnection,
                          name_or_id,
                          ignore_missing=ignore_missing)

    def update_aws_connection(self, connection, **params):
        """Update connection between ECL2.0 and AWS.

        :param string connection_id: The ID of a connection.
        :attrs \*\*params: Parameters for connection update.

            * string name: Name of connection.
            * string description: Description of connection.
            * string bandwidth: Bandwidth. If you selected BestEffort
                in the qos_type, can update 100 or 1000 Mbps.
                However, in case of Guarantee, this parameter can not update.
        """
        if not isinstance(connection, _aws_connection.AWSConnection):
            connection = self._get_resource(_aws_connection.AWSConnection,
                                            connection)
            connection._body.clean()
        return self._update(_aws_connection.AWSConnection,
                            connection, **params)

    def delete_aws_connection(self, connection_id, ignore_missing=False):
        """Delete connection between ECL2.0 and AWS.

        :param connection_id: The ID of a connection.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the port does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent port.

        :returns: ``None``
        """
        self._delete(_aws_connection.AWSConnection,
                     connection_id,
                     ignore_missing=ignore_missing)

    def prepare_aws_connection(self, aws_account_id, connected_network_address,
                           bandwidth, qos_type, exchange_point_id,
                           name=None, description=None):
        """Create connection between ECL2.0 and AWS.

        :param string aws_account_id: AWS Account ID of owner.
        :param string connected_network_address: Connected network address
            for AWS connectivity.(/28)
        :param string bandwidth: Bandwidth(Mbps). If you choose BestEffort in
            the qos_type, can select 100 or 1000.
            When you select Guarantee, choose from 50 to 500.
        :param string qos_type: QoS type of network.
            You can choose from "BestEffort" or "Guarantee".
        :param string exchange_point_id: ID of exchange_points.
        :param string name: Name of connection.
        :param string description: Description of connection.
        """
        body = {}
        body["aws_account_id"] = aws_account_id
        body["connected_network_address"] = connected_network_address
        body["bandwidth"] = bandwidth
        body["qos_type"] = qos_type
        body["exchange_point_id"] = exchange_point_id
        if name:
            body["name"] = name
        if description:
            body["description"] = description
        return self._create(_aws_connection.AWSConnection, **body)

    def approve_aws_connection(self, connection_id, action):
        """Approve/Disapprove connection between ECL2.0 and AWS.

        :param connection_id: ID of specified connection.
        :param type: Action type. You can choose "approve" or "disapprove".
        :return: ``None``
        """
        connection = _aws_connection.AWSConnection()
        connection.approve(self.session, connection_id, action)

    def exchange_points(self, **params):
        """List exchange ponts.

        :param params: The parameters as query string
            to get exchange points by specified condition.
        :returns: A list of exchange point objects
        :rtype: list of :class:`~ecl.provider_connectivity.v1.exchange_point.
            ExchangePoint`
        """
        return list(self._list(_exchange_point.ExchangePoint,
                               paginated=False,
                               **params))

    def operations(self, **params):
        """List operations.

        :param params: The parameters as query string
            to get operations by specified condition.
        :returns: A list of operation objects
        :rtype: list of :class:`~ecl.provider_connectivity.v1.operation.
            Operation`
        """
        return list(self._list(_operation.Operation,
                               paginated=False,
                               **params))

    def tenant_connection_requests(self, **query):
        """Return a list of tenant_connection_requests

        :param kwargs query: Query parameter to get tenant_connection_requests.

        :returns: A list of tenant_connection_requests objects
        """

        return list(self._list(
            _tc_request.TenantConnectionRequest, paginated=False, **query))

    def create_tenant_connection_request(self, keystone_user_id,
                                         tenant_id_other, tenant_id,
                                         network_id, **params):
        """Create a tenant_connection_request resource.

        :param keystone_user_id: Keystone User ID who can access to the
                        owner tenant of tenant_connection_request
        :param tenant_id_other: The owner tenant of network
        :param tenant_id: The owner tenant of tenant_connection_request
        :param network_id: Network unique id
        :param params:
                name: Name of tenant_connection_request.
                description: Description of tenant_connection_request.
                tags: tenant_connection_request tags.
        :return: One :class:
                `~ecl.provider_connectivity.v1.tenant_connection_request.
                    TenantConnectionRequest` or None
        """
        body = {
            "tenant_id_other": tenant_id_other,
            "network_id": network_id
        }
        if params.get("name"):
            body["name"] = params.get("name")
        if params.get("description"):
            body["description"] = params.get("description")
        if params.get("tags"):
            body["tags"] = params.get("tags")
        if params.get("keystone_user_id"):
            body["keystone_user_id"] = params.get("keystone_user_id")
        if params.get("tenant_id"):
            body["tenant_id"] = params.get("tenant_id")
        return self._create(_tc_request.TenantConnectionRequest, **body)

    def update_tenant_connection_request(self, tenant_connection_request,
                                         **params):
        """Update tenant_connection_request resource.

        :param tenant_connection_request: tenant_connection_request unique ID.
        :param params:
            name: Name of tenant_connection_request.
            description: Description of tenant_connection_request.
            tags: tenant_connection_request tags
            name_other: Name for the owner of network.
            description_other: Description for the owner of network.
            tags_other: tags for the owner of network.
        :return: One :class:
                `~ecl.provider_connectivity.v1.tenant_connection_request.
                    TenantConnectionRequest` or None
        """

        if not isinstance(tenant_connection_request,
                          _tc_request.TenantConnectionRequest):
            tenant_connection_request = self._get_resource(
                _tc_request.TenantConnectionRequest,
                tenant_connection_request)
            tenant_connection_request._body.clean()
        return self._update(_tc_request.TenantConnectionRequest,
                            tenant_connection_request, **params)

    def delete_tenant_connection_request(self, tenant_connection_request,
                                         ignore_missing=False):
        """Delete tenant_connection_request resource.

        :param tenant_connection_request: tenant_connection_request unique ID.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent port.
        :return: ``None``
        """
        self._delete(_tc_request.TenantConnectionRequest,
                     tenant_connection_request,
                     ignore_missing=ignore_missing)

    def get_tenant_connection_request(self, tenant_connection_request_id):
        """Get a single tenant connection request

        :param tenant_connection_request_id:
                The ID of a tenant connection request

        :returns: One :class:`~ecl.provider_connectivity.v1.
                tenant_connection_request.TenantConnectionRequest` or None
        """
        return self._get(_tc_request.TenantConnectionRequest,
                         tenant_connection_request_id)

    def find_tenant_connection_request(self, name_or_id, ignore_missing=False):
        """Find a single tenant connection request

        :param name_or_id: The name or ID of a tenant connection request.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    aittempting to find a nonexistent resource.
        :returns: One :class:`~ecl.provider_connectivity.v1.
                tenant_connection_request.TenantConnectionRequest` or None
        """
        return self._find(_tc_request.TenantConnectionRequest,
                          name_or_id,
                          ignore_missing=ignore_missing)

    def tenant_connections(self, **query):
        """Return a list of tenant_connection

        :param kwargs query: Query parameter to get tenant_connection

        :returns: A list of tenant_connection objects
        """
        return list(self._list(_tenant_connection.TenantConnection,
                               paginated=False, **query))

    def get_tenant_connection(self, tenant_connection_id):
        """Get a single tenant connection

        :param tenant_connection_id: The ID of a tenant connection
        :return: One :class:`~ecl.provider_connectivity.v1.
                tenant_connection_request.TenantConnection` or None

        """
        return self._get(_tenant_connection.TenantConnection,
                         tenant_connection_id)

    def create_tenant_connection(self, tenant_connection_request_id,
                                 device_type, device_id, **params):
        """Create tenant_connection resource.

        :param tenant_connection_request_id: tenant_connection_request
                                            unique ID.
        :param device_type: device type
        :param device_id: device unique ID
        :param params:
            device_interface_id: physical port unique ID in Baremetal server.
                        Required if ECL::Baremetal::Server
                        is selected with device_type.
            attachment_opts:
                segmentation_type: segmentation type used for port.
                        Optional if ECL::Baremetal::Server is
                        selected with device_type.
                segmentation_id: segmentation id used for port.
                        Optional if ECL::Baremetal::Server is
                        selected with device_type.
                fixed_ips: Array of IP address assignment objects,
                        attached to port.
                    ip_address: IP address assigned to port
                    subnet_id: The ID of subnet from which IP
                        address is allocated.
        :return: One :class:`~ecl.provider_connectivity.v1.
                tenant_connection.TenantConnection` or None
        """
        body = {
            "tenant_connection_request_id": tenant_connection_request_id,
            "device_type": device_type,
            "device_id": device_id,
        }
        if params.get("name"):
            body["name"] = params.get("name")
        if params.get("description"):
            body["description"] = params.get("description")
        if params.get("tags"):
            body["tags"] = params.get("tags")
        if params.get("device_interface_id"):
            body["device_interface_id"] = params.get("device_interface_id")
        if params.get("attachment_opts"):
            body["attachment_opts"] = params.get("attachment_opts")
        return self._create(_tenant_connection.TenantConnection, **body)

    def update_tenant_connection(self, tenant_connection, **params):
        """Update tenant_connection_resource

        :param tenant_connection:
        :param params:
            name: Name of tenant_connection.
            description: Description of tenant_connection.
            tags: tenant_connection tags
            name_other: Name for the owner of network.
            description_other: Description for the owner of network.
            tags_other: tags for the owner of network.
        :return: One :class:
                `~ecl.provider_connectivity.v1.tenant_connection.
                    TenantConnection` or None
        """
        if not isinstance(tenant_connection,
                          _tenant_connection.TenantConnection):
            tenant_connection = self._get_resource(
                _tenant_connection.TenantConnection,
                tenant_connection)
            tenant_connection._body.clean()

        return self._update(_tenant_connection.TenantConnection,
                            tenant_connection, **params)

    def delete_tenant_connection(self, tenant_connection,
                                 ignore_missing=False):
        """Delete tenant_connection resource.

        :param tenant_connection:tenant_connection unique ID.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent port.
        :return: ``None``
        """
        self._delete(_tenant_connection.TenantConnection,
                     tenant_connection,
                     ignore_missing=ignore_missing)

    def find_tenant_connection(self, name_or_id, ignore_missing=False):
        """Find a single tenant connection

        :param name_or_id: The name or ID of a tenant connection.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.provider_connectivity.v1.
                tenant_connection.TenantConnection` or None
        """
        return self._find(_tenant_connection.TenantConnection,
                          name_or_id,
                          ignore_missing=ignore_missing)

    def address_assignments(self, tenant_connection_request_id, **query):
        """Return a list of object containing IP address and mac address
            used by network.

        :param tenant_connection_request_id: tenant_connection_request ID.
        :param query: Query parameter to get address_assignment

        :return: A list of address_assignment objects
        """
        return list(self._list(
            _addr_assignment.AddressAssignment,
            paginated=False,
            tenant_connection_request_id=tenant_connection_request_id,
            **query))

    def get_icc_network(self, tenant_connection_request_id):
        """List network resource used by network.

        :param tenant_connection_request_id: tenant_connection_request ID.
        :param query: Query parameter to get network

        :return: A list of network objects
        """
        icc_network = _addr_assignment.ICCNetwork()
        return icc_network.get(self.session, tenant_connection_request_id)

    def icc_subnets(self, tenant_connection_request_id, **query):
        """List subnet resource used by network.

        :param tenant_connection_request_id: tenant_connection_request ID.
        :param query: Query parameter to get subnet

        :return: A list of subnet objects
        """
        return list(self._list(
            _addr_assignment.ICCSubnet,
            paginated=False,
            tenant_connection_request_id=tenant_connection_request_id,
            **query))

    def get_icc_subnet(self, tenant_connection_request_id, subnet_id):
        """Get subnet resource used by network.

        :param tenant_connection_request_id: tenant_connection_request ID.
        :param subnet_id: subnet ID
        :param query: Query parameter to get subnet

        :return: A list of subnet objects
        """
        return self._get(
            _addr_assignment.ICCSubnet,
            subnet_id,
            tenant_connection_request_id=tenant_connection_request_id
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.database import database_service
from ecl import resource2


class Flavor(resource2.Resource):
    resource_key = 'flavor'
    resources_key = 'flavors'
    base_path = '/flavors'
    service = database_service.DatabaseService()

    # capabilities
    allow_get = True
    allow_list = True

    _query_mapping = resource2.QueryParameters()

    # Properties
    #: Links pertaining to this flavor. This is a list of dictionaries,
    #: each including keys ``href`` and ``rel``.
    links = resource2.Body('links')
    #: The name of this flavor.
    name = resource2.Body('name')
    #: Size of the disk this flavor offers. *Type: int*
    disk = resource2.Body('disk', type=int)
    #: ``True`` if this is a publicly visible flavor. ``False`` if this is
    #: a private image. *Type: bool*
    is_public = resource2.Body('os-flavor-access:is_public', type=bool)
    #: The amount of RAM (in MB) this flavor offers. *Type: int*
    ram = resource2.Body('ram', type=int)
    #: The number of virtual CPUs this flavor offers. *Type: int*
    vcpus = resource2.Body('vcpus', type=int)
    #: Size of the swap partitions.
    swap = resource2.Body('swap')
    #: Size of the ephemeral data disk attached to this server. *Type: int*
    ephemeral = resource2.Body('ephemeral', type=int)
    #: ``True`` if this flavor is disabled, ``False`` if not. *Type: bool*
    is_disabled = resource2.Body('OS-FLV-DISABLED:disabled', type=bool)
    #: The bandwidth scaling factor this flavor receives on the network.
    rxtx_factor = resource2.Body('rxtx_factor', type=float)
    #: ID
    str_id = resource2.Body('str_id', alternate_id=True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import resource
from ecl.telemetry import telemetry_service


class Sample(resource.Resource):
    """.. caution:: This API is a work in progress and is subject to change."""
    id_attribute = 'sample_id'
    base_path = '/meters/%(counter_name)s'
    service = telemetry_service.TelemetryService()

    # Supported Operations
    allow_create = True
    allow_list = True

    # Properties
    #: The meter name this sample is for
    counter_name = resource.prop('meter', alias='counter_name')
    #: When the sample has been generated.
    generated_at = resource.prop('timestamp')
    #: Arbitrary metadata associated with the sample
    metadata = resource.prop('metadata', alias='resource_metadata')
    #: The ID of the project this sample was taken for
    project_id = resource.prop('project_id')
    #: When the sample has been recorded.
    recorded_at = resource.prop('recorded_at')
    #: The ID of the resource this sample was taken for
    resource_id = resource.prop('resource_id')
    #: The name of the source that identifies where the sample comes from
    source = resource.prop('source')
    #: The meter type
    type = resource.prop('type', alias='counter_type')
    #: The unit of measure
    unit = resource.prop('unit', alias='counter_unit')
    #: The ID of the user this sample was taken for
    user_id = resource.prop('user_id')
    #: The metered value
    volume = resource.prop('volume', alias='counter_volume')

    @classmethod
    def list(cls, session, limit=None, marker=None, path_args=None,
             paginated=False, **params):
        url = cls._get_url(path_args)
        resp = session.get(url, endpoint_filter=cls.service, params=params)
        for item in resp.json():
            yield cls.existing(**item)

    def create(self, session):
        url = self._get_url(self)
        # telemetry expects a list of samples
        attrs = self._attrs.copy()
        attrs.pop('meter', None)
        resp = session.post(url, endpoint_filter=self.service,
                            json=[attrs])
        resp = resp.json()
        self.update_attrs(**resp.pop())
        self._reset_dirty()
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.baremetal import baremetal_service
from ecl import resource2


class Metadata(resource2.Resource):
    resource_key = None
    resources_key = None
    base_path = '/servers/%(server_id)s/metadata'
    service = baremetal_service.BaremetalService()

    # Capabilities
    allow_list = True
    allow_show = True
    allow_create = True
    allow_delete = True

    # Properties
    #: Metadata of the server.
    metadata = resource2.Body("metadata")

    def list(self, session, server_id):

        uri = self.base_path % {"server_id" : server_id}
        resp = session.get(uri, endpoint_filter=self.service,
                           headers={"Accept": "application/json"})
        self._translate_response(resp, has_body=True)
        return self

    def get(self, session, server_id, key):
        uri = self.base_path + '/%(key)s'
        uri = uri % {"server_id":server_id, "key":key}
        resp = session.get(uri, endpoint_filter=self.service,
                           headers={"Accept": "application/json"})
        self._translate_response(resp, has_body=True)
        return self

    def delete(self, session, server_id, key):
        uri = self.base_path + '/%(key)s'
        uri = uri % {"server_id": server_id, "key": key}
        resp = session.delete(uri, endpoint_filter=self.service,
                              headers={"Accept": ""})
        self._translate_response(resp, has_body=False)
        return self

    def merge(self, session, server_id, **attrs):
        uri = self.base_path % {"server_id" : server_id}
        body = {"metadata": attrs}
        resp = session.post(
            uri, endpoint_filter=self.service,
            json=body,
            headers={"Accept": "application/json"}
        )
        self._translate_response(resp, has_body=True)
        return self

    def replace(self, session, server_id, **attrs):
        uri = self.base_path % {"server_id" : server_id}
        body = attrs
        resp = session.put(
            uri, endpoint_filter=self.service,
            json=body,
            headers={"Accept": "application/json"}
        )
        self._translate_response(resp, has_body=True)
        return self

    def update(self, session, server_id, key, **attrs):
        uri = self.base_path + '/%(key)s'
        uri = uri % {"server_id": server_id, "key": key}
        body = attrs
        resp = session.put(
            uri, endpoint_filter=self.service,
            json=body,
            headers={"Accept": "application/json"}
        )
        self._translate_response(resp, has_body=True)
        return self



# -*- coding: utf-8 -*-


from ecl.connectivity.v1 import _proxy
from ecl.connectivity.v1 import mcic
from ecl.tests.unit import test_proxy_base2


class TestConnectivityProxy(test_proxy_base2.TestProxyBase):
    def setUp(self):
        super(TestConnectivityProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_mcics(self):
        self.verify_list(self.proxy.mcics, mcic.MCIC,
                         paginated=False,
                         method_kwargs={"query": 1},
                         expected_kwargs={"query": 1})



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestFirewallInterface(base.BaseFunctionalTest):
    def test_01_list_interface(self):
        interfaces = list(self.conn.network.firewall_interfaces())
        interface = interfaces[0]
        self.assertIsInstance(interface.firewall_id, six.string_types)
        self.assertIsInstance(interface.id, six.string_types)
        self.assertIsInstance(interface.name, six.string_types)
        self.assertIsInstance(interface.status, six.string_types)
        self.assertIsInstance(interface.tenant_id, six.string_types)

    def test_02_show_interface(self):
        interface = self.conn.network.show_firewall_interface(
            "a96d13c3-6f82-4dee-9d8f-3dc3abdee81b"
        )
        self.assertIsInstance(interface.firewall_id, six.string_types)
        self.assertIsInstance(interface.id, six.string_types)
        self.assertIsInstance(interface.name, six.string_types)
        self.assertIsInstance(interface.status, six.string_types)
        self.assertIsInstance(interface.tenant_id, six.string_types)

    def test_03_update_interface(self):
        interface = self.conn.network.update_firewall_interface(
            "fca334de-0d13-4a94-ab3d-ef1b2d389573",
            description="new description"
        )
        self.assertIsInstance(interface.description, six.string_types)
        self.assertIsInstance(interface.firewall_id, six.string_types)
        self.assertIsInstance(interface.id, six.string_types)
        self.assertIsInstance(interface.name, six.string_types)
        self.assertIsInstance(interface.status, six.string_types)
        self.assertIsInstance(interface.tenant_id, six.string_types)


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import sys
import testtools

from ecl import utils


class Test_enable_logging(testtools.TestCase):

    def _console_tests(self, fake_logging, level, debug, stream):
        the_logger = mock.Mock()
        fake_logging.getLogger.return_value = the_logger

        utils.enable_logging(debug=debug, stream=stream)

        self.assertEqual(the_logger.addHandler.call_count, 2)
        the_logger.setLevel.assert_called_with(level)

    def _file_tests(self, fake_logging, level, debug):
        the_logger = mock.Mock()
        fake_logging.getLogger.return_value = the_logger
        fake_path = "fake/path.log"

        utils.enable_logging(debug=debug, path=fake_path)

        fake_logging.FileHandler.assert_called_with(fake_path)
        self.assertEqual(the_logger.addHandler.call_count, 2)
        the_logger.setLevel.assert_called_with(level)

    def test_none(self):
        self.assertRaises(
            ValueError, utils.enable_logging,
            debug=True, path=None, stream=None)

    @mock.patch("ecl.utils.logging")
    def test_debug_console_stderr(self, fake_logging):
        self._console_tests(fake_logging,
                            fake_logging.DEBUG, True, sys.stderr)

    @mock.patch("ecl.utils.logging")
    def test_warning_console_stderr(self, fake_logging):
        self._console_tests(fake_logging,
                            fake_logging.WARNING, False, sys.stderr)

    @mock.patch("ecl.utils.logging")
    def test_debug_console_stdout(self, fake_logging):
        self._console_tests(fake_logging,
                            fake_logging.DEBUG, True, sys.stdout)

    @mock.patch("ecl.utils.logging")
    def test_warning_console_stdout(self, fake_logging):
        self._console_tests(fake_logging,
                            fake_logging.WARNING, False, sys.stdout)

    @mock.patch("ecl.utils.logging")
    def test_debug_file(self, fake_logging):
        self._file_tests(fake_logging, fake_logging.DEBUG, True)

    @mock.patch("ecl.utils.logging")
    def test_warning_file(self, fake_logging):
        self._file_tests(fake_logging, fake_logging.WARNING, False)


class Test_urljoin(testtools.TestCase):

    def test_strings(self):
        root = "http://www.example.com"
        leaves = "foo", "bar"

        result = utils.urljoin(root, *leaves)
        self.assertEqual(result, "http://www.example.com/foo/bar")

    def test_with_none(self):
        root = "http://www.example.com"
        leaves = "foo", None

        result = utils.urljoin(root, *leaves)
        self.assertEqual(result, "http://www.example.com/foo/")



# -*- coding: utf-8 -*-

from ecl.security_portal.v1 import security_device as _sd
from ecl.security_portal.v1 import security_device_interface as _sdi
from ecl import proxy2


class Proxy(proxy2.BaseProxy):
    def security_devices(self):
        """Listing security devices associated with specific tenant.

        :return: List security devices.
        :rtype: :class:`~ecl.security_portal.v1.security_device.SecurityDevice`
        """
        return list(self._list(_sd.SecurityDevice, paginated=False,
                               tenantid=self.session.get_project_id(),
                               usertoken=self.session.get_token()))

    def get_security_device(self, server_id):
        """Show security device details associated with specific tenant.

        :param string server_id: Server ID registered in Openstack(UUID).
        :return: One security device.
        :rtype: :class:`~ecl.security_portal.v1.security_device.SecurityDevice`
        """
        sd = _sd.SecurityDevice()
        return sd.get(self.session, server_id)

    def security_device_interfaces(self, server_id):
        """Listing security device Interfaces associated with specific tenant.

        :param string server_id: Server ID registered in Openstack(UUID).
        :return: List security device interfaces.
        :rtype: :class:`~ecl.security_portal.v1.security_device_interface.SecurityDeviceInterface`
        """
        return list(self._list(_sdi.SecurityDeviceInterface, paginated=False,
                               server_id=server_id,
                               tenantid=self.session.get_project_id(),
                               usertoken=self.session.get_token()))

    def get_security_device_interface(self, port_id):
        """Show security device Interface associated with specific tenant.

        :param string port_id: Port ID registered in Openstack(UUID).
        :return: One security device interface.
        :rtype: :class:`~ecl.security_portal.v1.security_device_interface.SecurityDeviceInterface`
        """
        sdi = _sdi.SecurityDeviceInterface()
        return sdi.get(self.session, port_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base


class TestVolumeType(base.BaseFunctionalTest):
    def test_01_volume_types(self):
        types = list(self.conn.storage.volume_types(False))
        type = types[0]
        self.assertIsInstance(type.extra_specs, dict)
        self.assertIsInstance(type.id, six.string_types)
        self.assertIsInstance(type.name, six.string_types)

    def test_02_get_volume_type(self):
        type = self.conn.storage.get_volume_type(
            "6328d234-7939-4d61-9216-736de66d15f9"
        )
        self.assertIsInstance(type.extra_specs, dict)
        self.assertIsInstance(type.id, six.string_types)
        self.assertIsInstance(type.name, six.string_types)


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
The :class:`~ecl.session.Session` overrides
:class:`~keystoneauth1.session.Session` to provide end point filtering and
mapping KSA exceptions to SDK exceptions.

"""
import re
import json

from collections import namedtuple

from keystoneauth1 import exceptions as _exceptions
from keystoneauth1 import session as _session

from ecl import exceptions
from ecl import utils
from ecl import version as ecl_version

from ecl.baremetal import exceptions as baremetal_exp
from ecl.connectivity import exceptions as connectivity_exp
from ecl.dedicated_hypervisor import exceptions as dh_exp
from ecl.network import exceptions as network_exp
from ecl.rca import exceptions as rca_exp
from ecl.sss import exceptions as sss_exp
from ecl.storage import exceptions as storage_exp
from ecl.provider_connectivity import exceptions as icc_exp
from ecl.virtual_network_appliance import exceptions as vna_exp

from six.moves.urllib import parse

DEFAULT_USER_AGENT = "eclsdk/%s" % ecl_version.__version__
API_REQUEST_HEADER = "ecl-api-version"

Version = namedtuple("Version", ["major", "minor"])


def find_http_exception_class(e_url):
    mapper = {
        'baremetal-server': baremetal_exp.HttpException,
        'interconnectivity': connectivity_exp.HttpException,
        'dedicated-hypervisor': dh_exp.HttpException,
        'network': network_exp.HttpException,
        'rca': rca_exp.HttpException,
        'sss': sss_exp.HttpException,
        'storage': storage_exp.HttpException,
        'provider-connectivity': icc_exp.HttpException,
    }

    # to avoid matching to "network" endpoint.
    if re.search('virtual-network-appliance', e_url) is not None:
        return vna_exp.HttpException

    for pattern, exception_class in mapper.items():
        if re.search(pattern, e_url) is not None:
            return exception_class

    return exceptions.HttpException


def find_not_found_exception_class(e_url):
    mapper = {
        'baremetal-server': baremetal_exp.NotFoundException,
        'interconnectivity': connectivity_exp.NotFoundException,
        'dedicated-hypervisor': dh_exp.NotFoundException,
        'network': network_exp.NotFoundException,
        'rca': rca_exp.NotFoundException,
        'sss': sss_exp.NotFoundException,
        'storage': storage_exp.NotFoundException,
        'provider-connectivity': icc_exp.NotFoundException,
    }

    # to avoid matching to "network" endpoint.
    if re.search('virtual-network-appliance', e_url) is not None:
        return vna_exp.NotFoundException

    for pattern, exception_class in mapper.items():
        if re.search(pattern, e_url) is not None:
            return exception_class

    return exceptions.NotFoundException


def map_exceptions(func):
    def map_exceptions_wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except _exceptions.HttpError as e:
            if e.http_status == 404:
                exception_class = find_not_found_exception_class(e.url)
                exp = exception_class(
                    message=e.message,
                    details=e.details,
                    response=e.response,
                    request_id=e.request_id,
                    url=e.url,
                    method=e.method,
                    http_status=e.http_status,
                    cause=e)
                raise exp
            else:
                exception_class = find_http_exception_class(e.url)
                exp = exception_class(
                    message=e.message,
                    details=e.details,
                    response=e.response,
                    request_id=e.request_id,
                    url=e.url,
                    method=e.method,
                    http_status=e.http_status,
                    cause=e)
                raise exp
        except _exceptions.ClientException as e:
            raise exceptions.SDKException(message=e.message, cause=e)

    return map_exceptions_wrapper


class Session(_session.Session):

    def __init__(self, profile, user_agent=None, **kwargs):
        """Create a new Keystone auth session with a profile.

        :param profile: If the user has any special profiles such as the
            service name, region, version or interface, they may be provided
            in the profile object.  If no profiles are provided, the
            services that appear first in the service catalog will be used.
        :param user_agent: A User-Agent header string to use for the
                           request. If not provided, a default of
                           :attr:`~ecl.session.DEFAULT_USER_AGENT`
                           is used, which contains the eclsdk version
                           When a non-None value is passed, it will be
                           prepended to the default.
        :type profile: :class:`~ecl.profile.Profile`
        """
        if user_agent is not None:
            self.user_agent = "%s %s" % (user_agent, DEFAULT_USER_AGENT)
        else:
            self.user_agent = DEFAULT_USER_AGENT

        self.profile = profile
        api_version_header = self._get_api_requests()
        self.endpoint_cache = {}

        super(Session, self).__init__(user_agent=self.user_agent,
                                      additional_headers=api_version_header,
                                      **kwargs)

    def _get_api_requests(self):
        """Get API micro-version requests.

        :param profile: A profile object that contains customizations about
                        service name, region, version, interface or
                        api_version.
        :return: A standard header string if there is any specialization in
                 API microversion, or None if no such request exists.
        """
        if self.profile is None:
            return None

        req = []
        for svc in self.profile.get_services():
            if svc.service_type and svc.api_version:
                req.append(" ".join([svc.service_type, svc.api_version]))
        if req:
            return {API_REQUEST_HEADER: ",".join(req)}

        return None

    def _get_endpoint_versions(self, service_type, endpoint):
        """Get available endpoints from the remote service

        Take the endpoint that the Service Catalog gives us, then split off
        anything and just take the root. We need to make a request there
        to get the versions the API exposes.
        """
        parts = parse.urlparse(endpoint)
        root_endpoint = "://".join([parts.scheme, parts.netloc])
        response = self.get(root_endpoint)

        # Normalize the version response. Identity nests the versions
        # a level deeper than others, inside of a "values" dictionary.
        response_body = response.json()
        if "versions" in response_body:
            versions = response_body["versions"]
            if "values" in versions:
                versions = versions["values"]
            return root_endpoint, versions

        raise exceptions.EndpointNotFound(
            "Unable to parse endpoints for %s" % service_type)

    def _parse_version(self, version):
        """Parse the version and return major and minor components

        If the version was given with a leading "v", e.g., "v3", strip
        that off to just numerals.
        """
        version_num = version[version.find("v") + 1:]
        components = version_num.split(".")
        if len(components) == 1:
            # The minor version of a v2 ends up being -1 so that we can
            # loop through versions taking the highest available match
            # while also working around a direct match for 2.0.
            rv = Version(int(components[0]), -1)
        elif len(components) == 2:
            rv = Version(*[int(component) for component in components])
        else:
            raise ValueError("Unable to parse version string %s" % version)

        return rv

    def _get_version_match(self, versions, profile_version, service_type,
                           root_endpoint, requires_project_id):
        """Return the best matching version

        Look through each version trying to find the best match for
        the version specified in this profile.
         * The best match will only ever be found within the same
           major version, meaning a v2 profile will never match if
           only v3 is available on the server.
         * The search for the best match is fuzzy if needed.
           * If the profile specifies v2 and the server has
             v2.0, v2.1, and v2.2, the match will be v2.2.
           * When an exact major/minor is specified, e.g., v2.0,
             it will only match v2.0.
        """
        match = None
        for version in versions:
            api_version = self._parse_version(version["id"])
            if profile_version.major != api_version.major:
                continue

            if profile_version.minor <= api_version.minor:
                for link in version["links"]:
                    if link["rel"] == "self":
                        match = link["href"]

            # Only break out of the loop on an exact match,
            # otherwise keep trying.
            if profile_version.minor == api_version.minor:
                break

        if match is None:
            raise exceptions.EndpointNotFound(
                "Unable to determine endpoint for %s" % service_type)

        # Some services return only the path fragment of a URI.
        # If we split and see that we're not given the scheme and netloc,
        # construct the match with the root from the service catalog.
        match_split = parse.urlsplit(match)
        if not all([match_split.scheme, match_split.netloc]):
            match = root_endpoint + match

        # For services that require the project id in the request URI,
        # add them in here.
        if requires_project_id:
            match = utils.urljoin(match, self.get_project_id())

        return match

    def get_endpoint(self, auth=None, interface=None, service_type=None,
                     **kwargs):
        """Override get endpoint to automate endpoint filtering

        This method uses the service catalog to find the root URI of
        each service and then gets all available versions directly
        from the service, not from the service catalog.

        Endpoints are cached per service type and interface combination
        so that they're only requested from the remote service once
        per instance of this class.
        """
        key = (service_type, interface)
        if key in self.endpoint_cache:
            return self.endpoint_cache[key]

        filt = self.profile.get_filter(service_type)
        if filt.interface is None:
            filt.interface = interface

        sc_endpoint = super(Session, self).get_endpoint(auth,
                                                        **filt.get_filter())

        self.endpoint_cache[key] = sc_endpoint
        return sc_endpoint

        # Object Storage is, of course, different. Just use what we get
        # back from the service catalog as not only does it not offer
        # a list of supported versions, it appends an "AUTH_" prefix to
        # the project id so we'd have to special case that as well.
        # if service_type == "object-store":
        #     self.endpoint_cache[key] = sc_endpoint
        #     return sc_endpoint
        #
        # root_endpoint, versions = self._get_endpoint_versions(service_type,
        #                                                       sc_endpoint)
        # profile_version = self._parse_version(filt.version)
        # match = self._get_version_match(versions, profile_version,
        #                                 service_type, root_endpoint,
        #                                 filt.requires_project_id)
        #
        # self.endpoint_cache[key] = match
        # return match

    @map_exceptions
    def request(self, *args, **kwargs):
        return super(Session, self).request(*args, **kwargs)

    def _is_keystone_discover_result(self, content):
        """
        Check if given response body is keystone discover API result.
        :param string content: Response contents
        :return:
        """
        try:
            json_body = json.loads(content)
            if 'version' in json_body \
                    and 'links' in json_body['version'] \
                    and len(json_body['version']['links']) == 1 \
                    and 'href' in json_body['version']['links'][0]:
                return True
        except:
            return False
        return False

    def _replace_discoverd_keystone_url(self, content):
        """
        Replace discoverd url of keystone in case protocol returned as 'http'.
        Because only https access is allowd to ECL2.0 Keystone.
        :param string content: Response contents
        :return:
        """
        try:
            result = self._is_keystone_discover_result(content)
            if result:
                json_body = json.loads(content)

                # Check if url is returned as http url.
                discoverd_url = json_body['version']['links'][0]['href']
                if re.match('http:', discoverd_url):
                    https_url = discoverd_url.replace('http:', 'https:')
                    # Replace url if http.
                    json_body['version']['links'][0]['href'] = https_url

                    new_content = json.dumps(json_body)
                    return new_content
        except:
            pass
        return content

    def _send_request(self, url, method, redirect, log, logger,
                      connect_retries, connect_retry_delay=0.5, **kwargs):
        resp = super(Session, self)._send_request(
            url, method, redirect, log,
            logger, connect_retries,
            connect_retry_delay=connect_retry_delay,
            **kwargs)
        if re.match('https:', url):
            try:
                resp._content = self._replace_discoverd_keystone_url(
                    resp._content)
            except:
                pass
        return resp



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from ecl import exceptions
from ecl.image.v2 import image

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'id': IDENTIFIER,
    'checksum': '1',
    'container_format': '2',
    'created_at': '2015-03-09T12:14:57.233772',
    'data': 'This is not an image',
    'disk_format': '4',
    'min_disk': 5,
    'name': '6',
    'owner': '7',
    'properties': {'a': 'z', 'b': 'y', },
    'protected': False,
    'status': '8',
    'tags': ['g', 'h', 'i'],
    'updated_at': '2015-03-09T12:15:57.233772',
    'virtual_size': '10',
    'visibility': '11',
    'location': '12',
    'size': 13,
    'store': '14',
    'file': '15',
    'locations': ['15', '16'],
    'direct_url': '17',
    'path': '18',
    'value': '19',
    'url': '20',
    'metadata': {'21': '22'}
}


class TestImage(testtools.TestCase):

    def setUp(self):
        super(TestImage, self).setUp()
        self.resp = mock.Mock()
        self.resp.body = None
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.sess = mock.Mock()
        self.sess.post = mock.Mock(return_value=self.resp)

    def test_basic(self):
        sot = image.Image()
        self.assertIsNone(sot.resource_key)
        self.assertEqual('images', sot.resources_key)
        self.assertEqual('/v2/images', sot.base_path)
        self.assertEqual('image', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = image.Image(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['checksum'], sot.checksum)
        self.assertEqual(EXAMPLE['container_format'], sot.container_format)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['disk_format'], sot.disk_format)
        self.assertEqual(EXAMPLE['min_disk'], sot.min_disk)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['owner'], sot.owner)
        self.assertEqual(EXAMPLE['tags'], sot.tags)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)
        self.assertEqual(EXAMPLE['visibility'], sot.visibility)
        self.assertEqual(EXAMPLE['size'], sot.size)
        self.assertEqual(EXAMPLE['file'], sot.file)
        self.assertEqual(EXAMPLE['locations'], sot.locations)
        self.assertEqual(EXAMPLE['direct_url'], sot.direct_url)

    def test_upload(self):
        sot = image.Image(**EXAMPLE)

        self.assertIsNone(sot.upload(self.sess))
        self.sess.put.assert_called_with('images/IDENTIFIER/file',
                                         endpoint_filter=sot.service,
                                         data=sot.data,
                                         headers={"Content-Type":
                                                  "application/octet-stream",
                                                  "Accept": ""})

    def test_upload_checksum_match(self):
        sot = image.Image(**EXAMPLE)

        resp = mock.Mock()
        resp.content = b"abc"
        resp.headers = {"Content-MD5": "900150983cd24fb0d6963f7d28e17f72"}
        self.sess.get.return_value = resp

        rv = sot.download(self.sess)
        self.sess.get.assert_called_with('images/IDENTIFIER/file',
                                         endpoint_filter=sot.service)

        self.assertEqual(rv, resp.content)

    def test_upload_checksum_mismatch(self):
        sot = image.Image(**EXAMPLE)

        resp = mock.Mock()
        resp.content = b"abc"
        resp.headers = {"Content-MD5": "the wrong checksum"}
        self.sess.get.return_value = resp

        self.assertRaises(exceptions.InvalidResponse, 
                          sot.download, self.sess, "image-id")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from ecl.compute.v2 import server

IDENTIFIER = 'IDENTIFIER'

# NOTE: The implementation for metadata is done via a mixin class that both
# the server and image resources inherit from. Currently this test class
# uses the Server resource to test it. Ideally it would be parameterized
# to run with both Server and Image when the tooling for subtests starts
# working.


class TestMetadata(testtools.TestCase):

    def setUp(self):
        super(TestMetadata, self).setUp()
        self.metadata_result = {"metadata": {"go": "cubs", "boo": "sox"}}
        self.meta_result = {"meta": {"oh": "yeah"}}

    def test_get_all_metadata_Server(self):
        self._test_get_all_metadata(server.Server(id=IDENTIFIER))

    def test_get_all_metadata_ServerDetail(self):
        # This is tested explicitly so we know ServerDetail items are
        # properly having /detail stripped out of their base_path.
        self._test_get_all_metadata(server.ServerDetail(id=IDENTIFIER))

    def _test_get_all_metadata(self, sot):
        response = mock.Mock()
        response.json.return_value = self.metadata_result
        sess = mock.Mock()
        sess.get.return_value = response

        result = sot.get_metadata(sess)

        self.assertEqual(result, self.metadata_result["metadata"])
        sess.get.assert_called_once_with("servers/IDENTIFIER/metadata",
                                         headers={},
                                         endpoint_filter=sot.service)

    def test_set_metadata(self):
        response = mock.Mock()
        response.json.return_value = self.metadata_result
        sess = mock.Mock()
        sess.post.return_value = response

        sot = server.Server(id=IDENTIFIER)

        set_meta = {"lol": "rofl"}

        result = sot.set_metadata(sess, **set_meta)

        self.assertEqual(result, self.metadata_result["metadata"])
        sess.post.assert_called_once_with("servers/IDENTIFIER/metadata",
                                          endpoint_filter=sot.service,
                                          headers={},
                                          json={"metadata": set_meta})

    def test_delete_metadata(self):
        sess = mock.Mock()
        sess.delete.return_value = None

        sot = server.Server(id=IDENTIFIER)

        key = "hey"

        sot.delete_metadata(sess, [key])

        sess.delete.assert_called_once_with(
            "servers/IDENTIFIER/metadata/" + key,
            headers={"Accept": ""},
            endpoint_filter=sot.service)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from ecl.block_store.v2 import volume as _volume
from ecl.tests.functional import base


class TestVolume(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestVolume, cls).setUpClass()
        cls.test_name = "test_volume_sdk"
        volumes = cls.conn.block_store.volumes()
        cls.one_volume = None
        for vol in volumes:
            if vol.name == cls.test_name:
                cls.one_volume = vol
                break
        if cls.one_volume is None:
            cls.one_volume = cls.conn.block_store.create_volume(
                15, name=cls.test_name)

    # def test_list(self):
    #     sots = self.conn.block_store.volumes()
    #     for sot in sots:
    #         self.assertIsInstance(sot.id, six.string_types)
    #
    # def test_get(self):
    #     sot = self.conn.block_store.get_volume(self.one_volume.id)
    #     self.assertEqual(self.one_volume.id, sot.id)
    #     self.assertEqual(self.one_volume.name, sot.name)
    #
    # def test_create(self):
    #     sot = self.conn.block_store.create_volume(15, name=self.test_name)
    #     self.one_volume = sot
    #     vol = self.conn.block_store.get_volume(sot.id)
    #     self.assertEqual(vol.size, 15)
    #     self.assertIsInstance(vol.id, six.string_types)
    #     self.assertIsInstance(vol.name, six.string_types)
    #
    # def test_delete(self):
    #     ret = self.conn.block_store.delete_volume(self.one_volume)
    #     self.assertIsNone(ret)
    #
    # def test_update(self):
    #     self.conn.block_store.update_volume(self.one_volume.id,
    #                                         name="test_sdk_volume")
    #     self.assertEqual(self.one_volume.size, 15)
    #     self.assertIsInstance(self.one_volume.id, six.string_types)
    #     self.assertIsInstance(self.one_volume.name, six.string_types)
    #
    # def test_extend(self):
    #     vol = self.conn.block_store.extend_volume(self.one_volume.id, 1)
    #     self.assertIsInstance(vol.id, six.string_types)
    #
    # def test_upload2image(self):
    #     self.conn.block_store.upload_to_image(self.one_volume,
    #                                           image_name="test_sdk_vol2image")

    def test_update_bootable(self):
        self.conn.block_store.update_bootable(self.one_volume, True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.storage.v1 import snapshot as s

IDENTIFIER = 'IDENTIFIER'
BASIC_EXAMPLE = {
    'id': IDENTIFIER,
    "status": "available",
    "name": "snapshotForVol120160601",
    "description": None,
    "usage": 12,
    "volume_id": "90ff9a35-726d-47d6-a048-f080d2c90b57",
    "snapshot_type_id": "c113437f-755d-11e6-9494-080027aeede7",
    "created_at": "2015-05-17T18:14:34.000000",
    "updated_at": None,
    "error_message": ""
}


class TestSnapshot(testtools.TestCase):

    def test_basic(self):
        sot = s.Snapshot()
        self.assertEqual('snapshot', sot.resource_key)
        self.assertEqual('snapshots', sot.resources_key)
        self.assertEqual('/snapshots', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertEqual('storage', sot.service.service_type)

    def test_basic_detail(self):
        sot = s.SnapshotDetail()
        self.assertEqual('snapshot', sot.resource_key)
        self.assertEqual('snapshots', sot.resources_key)
        self.assertEqual('/snapshots/detail', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertEqual('storage', sot.service.service_type)

    def test_make_basic(self):
        sot = s.Snapshot(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['id'], sot.id)
        self.assertEqual(BASIC_EXAMPLE['status'], sot.status)
        self.assertEqual(BASIC_EXAMPLE['name'], sot.name)
        self.assertEqual(BASIC_EXAMPLE['description'], sot.description)
        self.assertEqual(BASIC_EXAMPLE['usage'], sot.usage)
        self.assertEqual(BASIC_EXAMPLE['volume_id'], sot.volume_id)
        self.assertEqual(BASIC_EXAMPLE['snapshot_type_id'], sot.snapshot_type_id)
        self.assertEqual(BASIC_EXAMPLE['description'], sot.description)
        self.assertEqual(BASIC_EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(BASIC_EXAMPLE['updated_at'], sot.updated_at)
        self.assertEqual(BASIC_EXAMPLE['error_message'], sot.error_message)



# -*- coding: utf-8 -*-

import six

from ecl.tests.functional import base


class TestMetadata(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestMetadata, cls).setUpClass()
        cls.test_name = "test-volume-sdk"
        volumes = cls.conn.block_store.volumes()
        cls.one_volume = None
        for vol in volumes:
            if vol.name == cls.test_name:
                cls.one_volume = vol
                break

    def _test_create(self):
        metadata = self.conn.block_store.create_metadata(self.one_volume.id,
                                                         key1="value1",
                                                         key2="value2")
        self.assertEqual(metadata.key1, "value1")
        self.assertEqual(metadata.key2, "value2")

    def test_update(self):
        metadata = self.conn.block_store.update_metadata(self.one_volume.id,
                                                         key1="value111",
                                                         key2="value222")
        self.assertEqual(metadata.key1, "value111")
        self.assertEqual(metadata.key2, "value222")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from ecl.identity import identity_service
from ecl import resource


class Trust(resource.Resource):
    resource_key = 'trust'
    resources_key = 'trusts'
    base_path = '/OS-TRUST/trusts'
    service = identity_service.IdentityService()

    # capabilities
    allow_create = True
    allow_delete = True
    allow_list = True
    allow_retrieve = True

    # Properties
    #: ID of the project upon which the trustor is
    #: delegating authorization. *Type: string*
    project_id = resource.prop('project_id')
    #: Specifies the expiration time of the trust. A trust may be revoked
    #: ahead of expiration. If the value represents a time in the past,
    #: the trust is deactivated.
    expires_at = resource.prop('expires_at')
    #: ID of the trust object. *Type: string*
    id = resource.prop('id')
    #: If ``impersonation`` is set to true, then the ``user`` attribute
    #: of tokens that are generated based on the trust will represent
    #: that of the trustor rather than the trustee, thus allowing the trustee
    #: to impersonate the trustor.
    #: If ``impersonation`` is set to ``False``, then the token's ``user``
    #: attribute will represent that of the trustee. *Type: bool*
    is_impersonation = resource.prop('impersonation', type=bool)
    #: Represents the user ID who is capable of consuming the trust.
    #: *Type: string*
    trustee_user_id = resource.prop('trustee_user_id')
    #: Represents the user ID who created the trust, and who's authorization is
    #: being delegated. *Type: string*
    trustor_user_id = resource.prop('trustor_user_id')
    #: Specifies the subset of the trustor's roles on the ``project_id``
    #: to be granted to the trustee when the token in consumed. The
    #: trustor must already be granted these roles in the project referenced
    #: by the ``project_id`` attribute. *Type: list*
    roles = resource.prop('roles')
    #: Redelegation count
    redelegation_count = resource.prop('redelegation_count')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.sss import  sss_service
from ecl import resource2


class IAMRole(resource2.Resource):
    resource_key = None
    resources_key = 'iam_roles'
    base_path = '/iam/roles'

    # Capabilities
    allow_create = True
    allow_delete = True
    allow_list = True
    allow_get = True

    # Properties
    #: The IAM Role's ID.
    iam_role_id = resource2.Body('iam_role_id', alternate_id=True)
    #: The IAM Role's name.
    iam_role_name = resource2.Body('iam_role_name')
    #: Description of the IAM Role.
    description = resource2.Body('description')
    #: Whitelist rules of API exectution.
    resources = resource2.Body('resources')
    #: The contact ID that the IAM Role belongs.
    contract_id = resource2.Body('contract_id')

    def list(self, session, contract_id):
        """List IAM Role list in the designated contract."""

        url = self.base_path + '?contract_id=%s' % contract_id
        resp = session.get(url, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self



# -*- coding: utf-8 -*-

from ecl.compute import compute_service
from ecl import resource2


class Volume(resource2.Resource):
    resource_key = 'volume'
    resources_key = 'volumes'
    base_path = '/os-volumes'
    service = compute_service.ComputeService()

    # capabilities
    allow_create = True
    allow_get = True
    allow_delete = True
    allow_list = True
    allow_update = True

    id = resource2.Body("id")
    name = resource2.Body("displayName")
    description = resource2.Body("displayDescription")
    size = resource2.Body("size")
    volume_type = resource2.Body("volumeType")
    metadata = resource2.Body("metadata", dict)
    availability_zone = resource2.Body("availabilityZone")
    snapshot_id = resource2.Body("snapshotId")
    attachments = resource2.Body("attachments", list)
    created_at = resource2.Body("createdAt")
    status = resource2.Body('status')
    bootable = resource2.Body('bootable')


class VolumeDetail(Volume):
    base_path = '/os-volumes/detail'

    allow_create = False
    allow_get = False
    allow_delete = False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.telemetry.v2 import meter

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'meter_id': IDENTIFIER,
    'name': 'instance',
    'project_id': '123',
    'resource_id': '456',
    'source': 'abc',
    'type': 'def',
    'unit': 'ghi',
    'user_id': '789'
}


class TestMeter(testtools.TestCase):

    def test_basic(self):
        sot = meter.Meter()
        self.assertEqual('meter', sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual('/meters', sot.base_path)
        self.assertEqual('metering', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_retrieve)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = meter.Meter(EXAMPLE)
        self.assertEqual(EXAMPLE['meter_id'], sot.id)
        self.assertEqual(EXAMPLE['meter_id'], sot.meter_id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['resource_id'], sot.resource_id)
        self.assertEqual(EXAMPLE['source'], sot.source)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['unit'], sot.unit)
        self.assertEqual(EXAMPLE['user_id'], sot.user_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.baremetal.v2 import flavor as _flavor
from ecl.baremetal.v2 import keypair as _keypair
from ecl.baremetal.v2 import limits as _limits
from ecl.baremetal.v2 import metadata as _metadata
from ecl.baremetal.v2 import uefi as _uefi
from ecl.baremetal.v2 import availability_zone as _zone
from ecl.baremetal.v2 import stock as _stock
from ecl.baremetal.v2 import nic_physical_port as _port
from ecl.baremetal.v2 import server as _server
from ecl.baremetal import version as _version
from ecl import proxy2
from ecl import session


class VersionSession(session.Session):

    def get_endpoint(self, auth=None, interface='public', service_type=None,
                     **kwargs):
        """Override get endpoint to automate endpoint filtering

        This method uses the service catalog to find the root URI of
        each service and then gets all available versions directly
        from the service, not from the service catalog.

        Endpoints are cached per service type and interface combination
        so that they're only requested from the remote service once
        per instance of this class.
        """

        sc_endpoint = super(VersionSession, self).get_endpoint(
            auth, interface, service_type
        )

        if sc_endpoint == None or sc_endpoint == '':
            return sc_endpoint
        segments = sc_endpoint.split('/')
        if len(segments) > 0:
            segments = segments[:-2]
            sc_endpoint = ""
            for segment in segments:
                sc_endpoint += segment + '/'
            sc_endpoint = sc_endpoint[:-1]

        return sc_endpoint


class Proxy(proxy2.BaseProxy):
    def availability_zones(self):
        """Return a list of availability zones for baremetal servers

        :returns: List of :class:`~ecl.baremetal.v2.availability_zone.AvailabilityZone`
        """
        return list(self._list(_zone.AvailabilityZone))

    def find_availability_zone(self, name_or_id, ignore_missing=False):
        """Find a single availability_zone

        :param string name_or_id: The name or ID of an availability zone.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: :class:`~ecl.baremetal.v2.availability_zone.AvailabilityZone` or ``None``
        """
        return self._find(_zone.AvailabilityZone, name_or_id,
                          ignore_missing=ignore_missing)

    def get_limits(self):
        """This API lists the current limits for the account

        :return: :class:`~ecl.baremetal.v2.limits.Limits`
        """

        limits = _limits.Limits()
        return limits.get(session=self.session)

    def flavors(self, details=True):
        """Lists all Flavor. That hash only id, links and name.
        If you want to get detail flavor information, call List Flavors
        Detail API. A flavor is a hardware configuration for a server.
        Each flavor is a unique combination of disk space and memory
        capacity.

        :return: A List of :class:`~ecl.baremetal.v2.flavor.Flavor`
        """
        flavor = _flavor.FlavorDetail if details else _flavor.Flavor
        return list(self._list(flavor))

    def get_flavor(self, flavor_id):
        """Gets details for a FlavorDetail associated with flavor_id.
        A flavor is a hardware configuration for a server.
        Each flavor is a unique combination of disk space and memory
        capacity.

        :param string flavor_id: ID of a flavor.
        :return: :class:`~ecl.baremetal.v2.flavor.Flavor`
        """
        return self._get(_flavor.Flavor, flavor_id)

    def find_flavor(self, name_or_id, ignore_missing=False):
        """Find a single flavor

        :param string name_or_id: Name or ID of a flavor.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :return: :class:`~ecl.baremetal.v2.flavor.Flavor` or ``None``
        """
        return self._find(_flavor.Flavor, name_or_id, ignore_missing=ignore_missing)

    def keypairs(self):
        """Lists name, public_key and finger_print for all KeyPairs.

        :return: A list of :class:`~ecl.baremetal.v2.keypair.Keypair`
        """
        keypairs = _keypair.Keypair()
        return list(keypairs.list(session=self.session))

    def get_keypair(self, keypair_name):
        """Show a KeyPair associated with keypair_name.

        :param string keypair: The name associated with a KeyPair.
        :return: :class:`~ecl.baremetal.v2.keypair.Keypair`
        """
        return self._get(_keypair.Keypair, keypair_name)

    def create_keypair(self, name, public_key=None):
        """Generate or import SSH keys. After generating a KeyPair,
        please specify the key_name parameter in the request body
        in Create Server request to associate the new server with SSH keys.

        :param string name: The name associated with a KeyPair.
        :param string public_key: Keypair's public key.
            If you do not specify this parameter,
            You can get the private key from Responce.
        :return: :class:`~ecl.baremetal.keypair.Keypair`
        """
        attrs = {}
        attrs["name"] = name
        if public_key:
            attrs["public_key"] = public_key
        return self._create(_keypair.Keypair, **attrs)

    def delete_keypair(self, keypair_name):
        """Delete KeyPair associated with keypair_name.

        :param string keypair_name: The name associated with a KeyPair.
        :return: ``None``
        """
        return self._delete(_keypair.Keypair, keypair_name)

    def get_uefi(self, server_id):
        """This API shows the UEFI setting information for specified Baremetal server.

        :param string server_id: ID for the requested server.
        :return: :class:`~ecl.baremetal.uefi.UEFI`
        """
        uefi = _uefi.UEFI()
        return uefi.get(self.session, server_id)

    def update_uefi(self, server_id, **attrs):
        """Updates UEFI setting of specified Baremetal server.
        This request will be accepted only when the task_state is None.

        :param string server_id: ID for the requested server.
        :param kwargs attrs: Attributes to be passed onto the
                           :meth:`~ecl.baremetal.uefi.UEFI.update`
                           method to be updated.
        :return: :class:`~ecl.baremetal.uefi.UEFI`
        """
        uefi = _uefi.UEFI()
        return uefi.update(self.session, server_id, **attrs)

    def get_version(self):
        """Show Baremetal Server API version.

        :return: :class:`~ecl.baremetal.version.Version`
        """
        version = _version.Version()
        v_session = VersionSession(
            profile=self.session.profile,
            user_agent=self.session.user_agent,
        )
        for attr, value in self.session.__dict__.iteritems():
            v_session.__setattr__(attr, value)
        return version.get_version(session=v_session)

    def versions(self):
        """List Baremetal Server API version.

        :return: A list of :class:`~ecl.baremetal.version.Version`
        """
        version = _version.Version()
        v_session = VersionSession(
            profile=self.session.profile,
            user_agent=self.session.user_agent,
        )
        for attr, value in self.session.__dict__.iteritems():
            v_session.__setattr__(attr, value)
        return list(version.list_version(session=v_session))

    def get_stock(self, flavor_id, availability_zone=None):
        """This API show which Baremetal server stock is available or not.

        :param string flavor_id: UUID of the flavor for search stock.
        :param string availability_zone: Name of the AvailabilityZone.
            If you specified this parameter, search stock from Baremetal
            server that matching AvailabilityZone and flavor_id.
            If you omit this parameter, stock search from all Baremetal
            server that matching flavor_id.
        :return: :class:`~ecl.baremetal.v2.stock.Stock`
        """
        stock = _stock.Stock()
        return stock.get(self.session, flavor_id, availability_zone=availability_zone)

    def get_nic_physical_port(self, server_id, port_id):
        """This API shows the specified NicPhysicalPort information for
        specified server.

        :param string server_id: ID for the specified server.
        :param string port_id: ID for the specified network interface port.
        :return: :class:`~ecl.baremetal.v2.nic_physical_port.NicPhysicalPort`
        """
        port = _port.NicPhysicalPort()
        return port.get(self.session, server_id, port_id)

    def nic_physical_ports(self, server_id):
        """This API lists all NicPhysicalPort information for the
        specified server.

        :param string server_id: ID for the specified server.
        :return: A list of :class:`~ecl.baremetal.v2.nic_physical_port.NicPhysicalPort`
        """
        port = _port.NicPhysicalPort()
        return list(port.list(self.session, server_id))

    def servers(self, details=True, **attrs):
        """This API lists your Baremetal servers information.

        :param bool details: When set to ``False``
                    :class:`~ecl.baremetal.v2.server.Server` instance
                    will be returned. The default, ``True``, will cause
                    :class:`~ecl.baremetal.v2.server.ServerDetail`
                    instances to be returned.
        :param attrs: Attributes to be passed onto the
                    :meth:`~ecl.baremetal.server.Server.list` method.
        :return: A list of :class:`~ecl.baremetal.v2.server.Server`
        """
        server = _server.ServerDetail if details else _server.Server
        return list(self._list(server, **attrs))

    def get_server(self, server_id):
        """Show your Baremetal server's information.

        :param string server_id: ID for specified server.
        :return: :class:`~ecl.baremetal.v2.server.Server`
        """
        return self._get(_server.Server, server_id)

    def find_server(self, name_or_id, ignore_missing=False):
        """Find a single baremetal server

        :param string name_or_id: Name or ID of a server.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :return: :class:`~ecl.baremetal.v2.server.Server` or ``None``
        """
        return self._find(_server.Server, name_or_id, ignore_missing=ignore_missing)

    def create_server(self, name, networks, flavor, admin_pass=None,
                      image=None, key_name=None, availability_zone=None,
                      user_data=None, raid_arrays=None,
                      lvm_volume_groups=None, filesystems=None,
                      metadata=None, personality=None):
        """This API create additional Baremetal server.

        :param string name: Name of your Baremetal server as a string.
        :param array networks: Network Array.
            If it is specified greater than two, default gateway is
            first network.
        :param string flavor: The flavor reference for the desired flavor for your
            Baremetal server.
        :param string admin_pass: Password for the administrator.
        :param string image: The image reference for the desired image for your
            Baremetal server.
        :param string key_name: SSH Keypair name you created on KeyPairs API.
        :param string availability_zone: The availability zone name in which to launch
            the server.
        :param string user_data: Configuration information or scripts to use upon
            launch. Must be Base64 encoded. Maximum size is 65535 bytes.
        :param array raid_arrays: Raid Arrays information.
        :param array lvm_volume_groups: LVM information. must be sure to specify
            if you have to true lvm parameters Raid Arrays.
        :param array filesystems: Partition filesystem / mount point information.
        :param dict metadata: Metadata key and value pairs. The maximum size of the
            metadata key and value is 255 bytes each.
        :param array personality: File path and contents (text only) to inject into
            your Baremetal server at launch. This parameter is not supported in
            esxi.
        :return: :class:`~ecl.baremetal.v2.server.Server`
        """
        attrs = {}
        attrs["name"] = name
        attrs["networks"] = networks
        attrs["flavorRef"] = flavor
        if admin_pass:
            attrs["adminPass"] = admin_pass
        if image:
            attrs["imageRef"] = image
        if key_name:
            attrs["key_name"] = key_name
        if availability_zone:
            attrs["availability_zone"] = availability_zone
        if user_data:
            attrs["user_data"] = user_data
        if raid_arrays:
            attrs["raid_arrays"] = raid_arrays
        if lvm_volume_groups:
            attrs["lvm_volume_groups"] = lvm_volume_groups
        if filesystems:
            attrs["filesystems"] = filesystems
        if metadata:
            attrs["metadata"] = metadata
        if personality:
            attrs["personality"] = personality
        server = _server.Server()
        return server.create(self.session, **attrs)

    def delete_server(self, server_id):
        """This API deletes a specified Baremetal server.

        :param string server_id: ID for the specified server.
        :return: ``None``
        """
        return self._delete(_server.Server, server_id)

    def start_server(self, server_id, boot_mode):
        """Power on the Baremetal Server associated with server_id.
        This request will be accepted only when the task_state is None.

        :param string server_id: ID for the server.
        :param string boot_mode: Baremetal Server boot mode.
            A valid value is DISK, PXE, LEGACY or ISO.
        :return: ``None``
        """
        server = _server.ServerAction()
        return server.start(self.session, server_id, boot_mode)

    def stop_server(self, server_id, type=None):
        """Stop the Baremetal Server associated with server_id.
        This request will be accepted only when the task_state is None.

        :param string server_id: ID for the server.
        :param string type: Baremetal Server shutdown mode.
            A valid value is HARD or SOFT.
        :return: ``None``
        """
        server = _server.ServerAction()
        return server.stop(self.session, server_id, type)

    def reboot_server(self, server_id, type, boot_mode):
        """Reboot the Baremetal Server associated with server_id.
        This request will be accepted only when the task_state is None.

        :param string server_id: ID for the server.
        :param string type: Baremetal Server shutdown mode.
            A valid value is HARD or SOFT. HARD is force restart by IPMI.
            This operations is equal to power down.
            SOFT is restated by ACPI.
        :param string boot_mode: Baremetal Server boot mode.
            A valid value is DISK, PXE, LEGACY or ISO.
        :return: ``None``
        """
        server = _server.ServerAction()
        return server.reboot(self.session, server_id, type, boot_mode)

    def media_attach(self, server_id, image):
        """Attach media to the Baremetal Server associated with server_id.
        This request will be accepted only when the task_state is None.

        :param string server_id: ID for the server.
        :param string image: ID of the image you want to attach.
        :return: ``None``
        """
        server = _server.ServerAction()
        return server.media_attach(self.session, server_id, image)

    def media_detach(self, server_id, image):
        """Detach media from the Baremetal Server associated with server_id.
        This request will be accepted only when the task_state is None.

        :param string server_id: ID for the server.
        :param string image: ID of the image you want to detach.
        :return: ``None``
        """
        server = _server.ServerAction()
        return server.media_detach(self.session, server_id, image)

    def get_management_console(self, server_id):
        """Gets information to access the remote console of Baremetal
        Server. This request will be accepted only when the task_state
        is None.
        This API response console parameters are used at Remote Console
        Access - Get Started. To access your Baremetal Server, you have
        to create VPN user and connect to VPN.

        :param string server_id: ID for the server.
        :return: :class:`~ecl.baremetal.v2.server.ServerAction`
        """
        server = _server.ServerAction()
        return server.get_management_console(self.session, server_id)

    def metadata(self, server_id):
        """This API lists metadata for a specified server.

        :param string server_id: ID for specified server.
        :return: A list of :class:`~ecl.baremetal.v2.metadata.Metadata`
        """
        metadata = _metadata.Metadata()
        return list(metadata.list(self.session, server_id))

    def show_metadata(self, server_id, key):
        """This API shows metadata item (key and value) by specifying key
        for a specified server.

        :param string server_id: ID for specified server.
        :param string key: A string. Maximum length is 255 characters.
        :return: :class:`~ecl.baremetal.v2.metadata.Metadata`
        """
        metadata = _metadata.Metadata()
        return metadata.get(self.session, server_id, key)

    def delete_metadata(self, server_id, key):
        """This API deletes metadata item (key and value) by specifying
        key for a specified server.

        :param string server_id: ID for specified server.
        :param string key: A string. Maximum length is 255 characters.
        :return: ``None``
        """
        metadata = _metadata.Metadata()
        return metadata.delete(self.session, server_id, key)

    def merge_metadata(self, server_id, **attr):
        """This API creates or merges metadata for a specified server.
        This API replaces items which match the specified keys in the
        request. This API does not remove items, unlike Replace Server
        Metadata Items.

        :param string server_id: ID for specified server.
        :param kwargs attr: Keyword arguments which will be used to do update
                     process for :class:`~ecl.baremetal.metadata.Metadata`.
        :return: :class:`~ecl.baremetal.v2.metadata.Metadata`
        """
        metadata = _metadata.Metadata()
        return metadata.merge(self.session, server_id, **attr)

    def replace_metadata(self, server_id, **attr):
        """This API creates or replaces metadata for a specified server.
        All existing metadata items are removed and completely replaced
        by the metadata items in the request. If you don't want to remove
        existing items, please use Merge Server Metadata Items.

        :param string server_id: ID for specified server.
        :param kwargs attr: Keyword arguments which will be used to do update
                     process for :class:`~ecl.baremetal.metadata.Metadata`.
        :return: :class:`~ecl.baremetal.v2.metadata.Metadata`
        """
        metadata = _metadata.Metadata()
        return metadata.replace(self.session, server_id, **attr)

    def update_metadata(self, server_id, key, **attr):
        """This API sets additonal metadata item (key and value) by
        specifying key for a specified server.

        :param string server_id: ID for specified server.
        :param string key: A string. Maximum length is 255 characters.
        :param kwargs attr: Keyword arguments which will be used to do update
                     process for :class:`~ecl.baremetal.metadata.Metadata`.
        :return: :class:`~ecl.baremetal.v2.metadata.Metadata`
        """
        metadata = _metadata.Metadata()
        return metadata.update(self.session, server_id, key, **attr)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestImage(base.BaseFunctionalTest):
    def test_01_list_image(self):
        images = list(self.conn.image.images(limit=5))
        image = images[0]
        self.assertIsInstance(image.name, six.string_types)
        self.assertIsInstance(image.image_status, six.string_types)
        self.assertIsInstance(image.tags, list)
        self.assertIsInstance(image.container_format, six.string_types)
        self.assertIsInstance(image.created_at, six.string_types)
        self.assertIsInstance(image.disk_format, six.string_types)
        self.assertIsInstance(image.locations, list)
        #self.assertIsInstance(image.direct_url, six.string_types)
        self.assertIsInstance(image.updated_at, six.string_types)
        self.assertIsInstance(image.visibility, six.string_types)
        self.assertIsInstance(image.min_disk, int)
        self.assertIsInstance(image.protected, bool)
        self.assertIsInstance(image.id, six.string_types)
        self.assertIsInstance(image.file, six.string_types)
        #self.assertIsInstance(image.checksum, six.string_types)
        self.assertIsInstance(image.owner, six.string_types)
        #self.assertIsInstance(image.size, int)
        self.assertIsInstance(image.min_ram, int)
        #self.assertIsInstance(image.image_schema, six.string_types)
        self.assertIsInstance(image.images_schema, six.string_types)
        self.assertIsInstance(image.first, six.string_types)

        #print images
        assert False

    @classmethod
    def test_02_create_image(cls):
        img = cls.conn.image.create_image(
            name="TEST_IMAGE_NAME",
            disk_format='raw',
            container_format='bare',
            properties='{"description": "This is not an image"}',
            #data=open('CONTRIBUTING.rst', 'r')
        )
        cls.id = img.id
        print img
    # id=5e9a2627-2acf-4ebf-b210-ffa75de9c1be

    def test_03_update_image(self):
        img = self.conn.image.update_image(
            "1b782d43-4e43-4d96-883b-ff423f8b8b7c",
            [{"path":"/name", "value":"sdk_test", "op":"replace"}]
        )
        print img

    def test_04_show_image(self):
        image = self.conn.image.get_image(
            "1b782d43-4e43-4d96-883b-ff423f8b8b7c")
        print image.response


    def test_05_delete_image(self):
        self.conn.image.delete_image(
            "5e9a2627-2acf-4ebf-b210-ffa75de9c1be")

    def test_06_upload_image(self):
        self.conn.image.upload_image(
            #self.id,
            "fake_id",
            "raw_data"
        )

    def test_07_download_image(self):
        image = self.conn.image.download_image(
            "1b782d43-4e43-4d96-883b-ff423f8b8b7c"
        )
        print image





# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.sss.v1 import user as _user
from ecl.sss.v1 import tenant as _tenant
from ecl.sss.v1 import role as _role
from ecl.sss.v1 import api_key as _api_key
from ecl.sss.v1 import channel as _channel
from ecl.sss.v1 import contract as _contract
from ecl.sss.v1 import iam_group as _iam_group
from ecl.sss.v1 import iam_role as _iam_role
from ecl import proxy2


class Proxy(proxy2.BaseProxy):
    def users(self):
        """
        List users in the designated contract.

        :return: A generator of user instances.
        :rtype: :class:`~ecl.sss.v1.user.User`
        """
        return list(self._list(_user.User, paginated=False))

    def find_user(self, user_id, ignore_missing=False):
        """
        Find a user. Get user information such as login id, email, apikey, etc.

        :param user_id: ID of a user
        :param ignore_missing: When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :return: One :class:`~ecl.sss.v1.user.User` or None
        """
        return self._find(_user.User, user_id,
                           ignore_missing=ignore_missing)

    def get_user(self, user):
        """
        Get user information such as login id, email, apikey, etc.

        :param user: ID of a user or a :class:`~ecl.sss.v1.user.User` instance.
        :return: One :class:`~ecl.sss.v1.user.User` or
                     :class:`~ecl.exceptions.ResourceNotFound`when no
                     resource can be found.
        """
        return self._get(_user.User, user)

    def create_user(self, login_id, mail_address, notify_password,
                    password=None):
        """
        Create user in the designated contract. Only supser user (contract
        owner user) allowed.

        :param login_id: Login id of new user.
        :param mail_address: Mail address of new user.
        :param notify_password: If this flag is set 'true',
            notification email will be sent to new users email
        :param password: Initial password of new user.
            If this parameter is not designated, random initial password is
            generated and applied to new user.
        :return: The results of user creation
        :rtype: :class:`~ecl.sss.v1.user.User`
        """
        body = {}
        body["login_id"] = login_id
        body["mail_address"] = mail_address
        body["notify_password"] = notify_password
        if password:
            body["password"] = password
        return self._create(_user.User, **body)

    def update_user(self, user_id, login_id=None, mail_address=None,
                    new_password=None):
        """
        Update user's informain.

        :param user_id: Specified user_id.
        :param login_id: Login id of new user.
        :param mail_address: Mail address of new user.
        :param new_password: New password of the user.
        :return: ``None``
        """
        body = {}
        if login_id:
            body["login_id"] = login_id
        if mail_address:
            body["mail_address"] = mail_address
        if new_password:
            body["new_password"] = new_password
        user = _user.User()
        return user.update(self.session, user_id, **body)

    def delete_user(self, user, ignore_missing=False):
        """
        Delete user. Only supser user (contract owner user) allowed. Contract
        owner user itself cannot be deleted

        :param user: Delete target user's user id.
        :param ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the user does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent server
        :return: ``None``
        """
        self._delete(_user.User, user, ignore_missing=ignore_missing)

    def tenants(self, **query):
        """
        List tenants in the designated contract.

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :return: A generator of tenant instances.
        :rtype: :class:`~ecl.sss.v1.tenant.Tenant`
        """
        return list(self._list(_tenant.Tenant, paginated=False, **query))

    def find_tenant(self, tenant_id, ignore_missing=False):
        """
        Find a tenant. Get tenant information such as tenant name, etc.

        :param tenant_id: ID of a tenant
        :param ignore_missing: When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :return: One :class:`~ecl.sss.v1.tenant.Tenant` or None
        """
        return self._find(_tenant.Tenant, tenant_id,
                           ignore_missing=ignore_missing)

    def create_tenant(self, **attrs):
        """
        Create new tenant in the designated contract. Only supser user
        (contract owner user) allowed.

        :param attrs: Keyword arguments which will be used to create
                      a :class:`~ecl.sss.v1.tenant.Tenant`,
                      comprised of the properties on the Tenant class.
        :return: The results of tenant creation
        :rtype: :class:`~ecl.sss.v1.tenant.Tenant`
        """
        return self._create(_tenant.Tenant, **attrs)

    def delete_tenant(self, tenant, ignore_missing = True):
        """
        Delete tenant. Only supser user (contract owner user) allowed.

        :param tenant: Delete target tenant's tenant id.
        :param ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the user does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent server
        :return: ``None``
        """
        self._delete(_tenant.Tenant, tenant, ignore_missing=ignore_missing)

    def create_role(self, user_id, tenant_id):
        """
        Create role between a user and a tenant. With role, users can access
        to the tenants.

        :param user_id: User which have new role.
        :param tenant_id: Tenant which the user have role.
        :return: :class:`~ecl.sss.v1.role.Role`
        """
        body = {}
        body["user_id"] = user_id
        body["tenant_id"] = tenant_id
        return self._create(_role.Role, **body)

    def delete_role(self, tenant_id, user_id):
        """
        Delete role between user and tenant. Contract owner user always have
        role to all tenants in the contract. Only supser user
        (contract owner user) allowed.

        :param tenant_id: Delete target tenant's tenant id.
        :param user_id: User's id of the role
        :return: ``None``
        """
        role = _role.Role()
        return role.delete(session=self.session,
                                 tenant_id=tenant_id, user_id=user_id)

    def update_api_key(self, user_id):
        """
        Update API key pair of target user.

        :param user_id: Change target user's user id.
        :return: The attributes to update on the api_key represented by
                ``Api_key``.
        """
        key = _api_key.Api_key()
        return key.update(session=self.session, user_id=user_id)

    def channels(self, get_contracts, **query):
        """
        Getting lists of information about the own channel and end user channel
        that own contract belongs. Only partner user allowed.

        :param get_contracts: The flag of whether getting contracts
                              (true or false(default)).
        :return: A generator of channels instances.
        :rtype: :class:`~ecl.sss.v1.channel.Channel`
        """
        return list(self._list(_channel.Channel, paginated=False,
                               get_contracts=get_contracts, **query))

    def create_contract(self, login_id, mail_address, channel_id, password=None,
                        external_reference_id=None, notify_password=None):
        """
        Create contract in designated channel. Only partner user allowed.

        :param login_id: Login ID of new user.
        :param mail_address: E-mail address of the user.
        :param channel_id: The channel means the group to manage contracts.
            The partner user will be given 2 channels. One is the channel that
            contains own contract. The other is the channel that contains all
            end user contracts which the partner user has. By executing the List
            Channel API(For partner user only),
            the user can get your (and enduser's) channel ID.
        :param password: Password of the user. If the API user set this item as
            blank, the system set initial random password automatically.
        :param external_reference_id: By using this item, the partner API user
            can associate optional string to the constract(e.g. The end user
            management ID in the partner user's system). Note that this ID will
            be NOT used to control the contract in ECL2.0 internal system.
            If the item is set as blank, ECL 2.0 system set the end user's
            contract ID automatically(e.g. econXXXXXXXX).
        :param notify_password: Setting true or false(default is false).
            This item designate whether the system should send to the login_ID
            and Password with e-mail.
        :return: :class:`~ecl.sss.v1.contract.Contract`
        """
        body = {}
        body["login_id"] = login_id
        body["mail_address"] = mail_address
        body["channel_id"] = channel_id
        if password:
            body["password"] = password
        if external_reference_id:
            body["external_reference_id"] = external_reference_id
        if notify_password:
            body["notify_password"] = notify_password
        return self._create(_contract.Contract, **body)

    def contracts(self, channel_id, include_deleted="false"):
        """
        List Contracts in the designated channel. Only partner user allowed.

        :param channel_id: Target channel_id under own contract.
        :param include_deleted: Setting true or false(default is false)
        (true : Include deleted contract/ false: Not include deleted contract.
        :return: A generator of contracts instances.
        :rtype: :class:`~ecl.sss.v1.contract.Contract`
        """
        contract = _contract.Contract()
        return list(contract.list(session=self.session, channel_id=channel_id,
                          include_deleted=include_deleted))

    def delete_contract(self, contract_id, ignore_missing=False):
        """
        Delete contract in designated channel. Only partner user allowed.

        :param contract_id: Contract ID of Delete target
        :param ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the user does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent server
        :return: ``None``
        """
        return self._delete(_contract.Contract, contract_id,
                            ignore_missing=ignore_missing)

    def get_contract(self, contract_id):
        """
        Get the information of the designated contract.

        :param contract_id: The contract ID getting more information.
        :return: One :class:`~ecl.sss.v1.contract.Contract`
        """
        return self._get(_contract.Contract, contract_id)

    def get_billing_info(self, channel_id, target_month):
        """
        Get billing statement of designated month.

        :param channel_id: Billing statement owner contract.
        :param target_month: Target billing month with YYYY-MM format
        :return: One :class:`~ecl.sss.v1.contract.Contract`
        """
        bill_info = _contract.Contract()
        return bill_info.get_billing_info(
            session=self.session, channel_id=channel_id,
            target_month=target_month
        )

    def get_monthly_billing_of_each_contract(self, contract_id, target_month,
                                             target_contract_id):
        """
        Get montly billing for each contract. Only partner user allowed.

        :param contract_id: Contract ID of this request.
        :param target_month: Target billing month with YYYY-MM format
        :param target_contract_id: Target contract ID
        :return:
        """
        bill_info = _contract.Contract()
        return bill_info.get_monthly_billing_of_each_contract(
            session=self.session,
            contract_id=contract_id, target_month=target_month,
            target_contract_id=target_contract_id
        )

    def iam_groups(self, contract_id):
        """
        List iam groups by contract id.

        :param contract_id: Contract ID ( Default is the Contract ID of
            API executing user ) .
        :return: A list of iam groups.
        :rtype: :class:`~ecl.sss.v1.iam_group.IAMGroup`
        """
        iam_group = _iam_group.IAMGroup()
        return list(iam_group.list(session=self.session,
                                   contract_id=contract_id))

    def create_iam_group(self, iam_group_name, contract_id=None,
                         description=None):
        """
        Create iam group.

        :param iam_group_name: New IAM Group name.
            Alphanumeric characters and @, -, ., _ can be used.
        :param contract_id: Contract ID ( Default is the Contract ID of API
            executing user ) .
        :param description: Description of the IAM Group.
        :return: :class:`~ecl.sss.v1.iam_group.IAMGroup`
        """
        body = {}
        body["iam_group_name"] = iam_group_name
        if contract_id:
            body["contract_id"] = contract_id
        if description:
            body["description"] = description
        return self._create(_iam_group.IAMGroup, **body)

    def delete_iam_group(self, iam_group_id, ignore_missing=False):
        """
        Delete iam group.

        :param iam_group_id: Group ID that you want to delete.
        :param ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the user does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent server
        :return: ``None``
        """
        return self._delete(_iam_group.IAMGroup, iam_group_id,
                            ignore_missing=ignore_missing)

    def assign_iam_role(self, iam_group_id, iam_role_id):
        """
        Assignment of the IAM Role to the IAM Group.

        :param iam_group_id: IAM Group ID that assign to the Role.
        :param iam_role_id: IAM Role ID that assign to the Group.
        :return: :class:`~ecl.sss.v1.iam_group.IAMGroup`
        """
        iam_group = _iam_group.IAMGroup()
        return iam_group.assign_iam_role(session=self.session,
                                         iam_group_id=iam_group_id,
                                         iam_role_id=iam_role_id)

    def delete_assign_iam_role(self, iam_group_id, iam_role_id):
        """
        Delete Assignment of the IAM Role to the IAM Group.

        :param iam_group_id: IAM Group ID.
        :param iam_role_id: IAM Role ID.
        :return: ``None``
        """
        iam_group = _iam_group.IAMGroup()
        return iam_group.delete_assign_iam_role(session=self.session,
                                                iam_group_id=iam_group_id,
                                                iam_role_id=iam_role_id)

    def users_designated_iam_group(self, iam_group_id):
        """
        List User list in the designated IAM Group ID.

        :param iam_group_id: IAM Group ID that you want to show user list.
        :return: :class:`~ecl.sss.v1.iam_group.IAMGroup`
        """
        iam_group = _iam_group.IAMGroup()
        return iam_group.list_users(session=self.session,
                                    iam_group_id=iam_group_id)

    def assign_user(self, iam_group_id, user_id):
        """
        Assignment of the IAM User to the IAM Group.

        :param iam_group_id: IAM Group ID that assign to the User.
        :param user_id: User ID that assign to the Group.
        :return: :class:`~ecl.sss.v1.iam_group.IAMGroup`
        """
        iam_group = _iam_group.IAMGroup()
        return iam_group.assign_user(session=self.session,
                                     iam_group_id=iam_group_id,
                                     user_id=user_id)

    def delete_assign_user(self, iam_group_id, user_id):
        """
        Delete Assignment of the User to the IAM Group.

        :param iam_group_id: IAM Group ID.
        :param user_id: User ID.
        :return: ``None``
        """
        iam_group = _iam_group.IAMGroup()
        return iam_group.delete_assign_user(session=self.session,
                                            iam_group_id=iam_group_id,
                                            user_id=user_id)

    def iam_roles(self, contract_id):
        """
        List IAM Role list in the designated contract.

        :param contract_id: Contract ID ( Default is the Contract ID of API
            executing user ) .
        :return: A list of iam groups.
        :rtype: :class:`~ecl.sss.v1.iam_role.IAMRole`
        """
        iam_role = _iam_role.IAMRole()
        return list(iam_role.list(session=self.session,
                                  contract_id=contract_id))

    def get_iam_role(self, iam_role_id):
        """
        Show a IAM Role's information. You can check a IAM Role's
            name, ID and resources by executing this API.

        :param iam_role_id: IAM Role ID that you want to check information.
        :return: :class:`~ecl.sss.v1.iam_role.IAMRole`
        """
        return self._get(_iam_role.IAMRole, iam_role_id)

    def create_iam_role(self, iam_role_name, resources, contract_id=None,
                        description=None):
        """
        Create IAM Role in the designated contract. Only supser user
        (contract owner user) can use this API.

        :param iam_role_name: New IAM Role name.
            Alphanumeric characters and @, -, ., _ can be used.
        :param resources: Whitelist rules of API execution.
            ( resources item is mandatory, but a user can define any rule by
            using follow parameters. )
        :param contract_id: Contract ID ( Default is the Contract ID of API
            executing user ).
        :param description: Description of the IAM Role.
        :return: :class:`~ecl.sss.v1.iam_group.IAMGroup`
        """
        body = {}
        body["iam_role_name"] = iam_role_name
        body["resources"] = resources
        if contract_id:
            body["contract_id"] = contract_id
        if description:
            body["description"] = description
        return self._create(_iam_role.IAMRole, **body)

    def delete_iam_role(self, iam_role_id, ignore_missing=False):
        """
        Delete designated IAM Role. Only supser user (contract owner user)
        can use this API.

        :param iam_role_id: IAM Role ID that you want to delete.
        :return: ``None``
        """
        return self._delete(_iam_role.IAMRole, iam_role_id,
                            ignore_missing=ignore_missing)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.compute.v2 import availability_zone as az

IDENTIFIER = 'IDENTIFIER'
BASIC_EXAMPLE = {
    'id': IDENTIFIER,
    'zoneState': 'available',
    'hosts': 'host1',
    'zoneName': 'zone1'
}


class TestAvailabilityZone(testtools.TestCase):

    def test_basic(self):
        sot = az.AvailabilityZone()
        self.assertEqual('availabilityZoneInfo', sot.resources_key)
        self.assertEqual('/os-availability-zone', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertEqual('compute', sot.service.service_type)

    def test_basic_detail(self):
        sot = az.AvailabilityZoneDetail()
        self.assertEqual('availabilityZoneInfo', sot.resources_key)
        self.assertEqual('/os-availability-zone/detail', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertEqual('compute', sot.service.service_type)

    def test_make_basic(self):
        sot = az.AvailabilityZone(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['id'], sot.id)
        self.assertEqual(BASIC_EXAMPLE['zoneState'], sot.state)
        self.assertEqual(BASIC_EXAMPLE['hosts'], sot.hosts)
        self.assertEqual(BASIC_EXAMPLE['zoneName'], sot.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import  testtools
from ecl.sss.v1 import tenant

BASIC_EXAMPLE = {
    "tenant_id": "91b5c7c09cbd45cab3c146fd67ae471e",
    "tenant_name": "new_project",
    "description": "cheeseshop web",
    "region": "jp1",
    "contract_id": "econ123467890"
}

class TestRole(testtools.TestCase):

    def test_role(self):
        sot = tenant.Tenant()
        self.assertEqual("tenants", sot.resources_key)
        self.assertEqual(None, sot.resource_key)
        self.assertEqual("/tenants", sot.base_path)
        self.assertEqual('sss', sot.service.service_type)

        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.put_create)

    def test_make_role(self):
        sot = tenant.Tenant(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['tenant_id'], sot.tenant_id)
        self.assertEqual(BASIC_EXAMPLE['tenant_name'], sot.tenant_name)
        self.assertEqual(BASIC_EXAMPLE['description'], sot.description)
        self.assertEqual(BASIC_EXAMPLE['region'], sot.region)
        self.assertEqual(BASIC_EXAMPLE['contract_id'], sot.contract_id)



# -*- coding: utf-8 -*-

from ecl.network import network_service
from ecl import resource2


class Quota(resource2.Resource):
    resource_key = 'quota'
    resources_key = 'quotas'
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/quotas'

    # capabilities
    allow_get = True
    allow_update = True
    allow_delete = True
    allow_list = True

    # Properties
    #: The maximum amount of colocation logical link you can have. *Type: int*
    colocation_logical_link = resource2.Body('colocation_logical_link', type=int)
    #: The maximum amount of common function gateway you can create. *Type: int*
    common_function_gateway = resource2.Body('common_function_gateway', type=int)
    #: The maximum amount of firewall you can create. *Type: int*
    firewall = resource2.Body('firewall', type=int)
    #: ID of quota
    id = resource2.Body("id")
    #: The maximum amount of interdc gateway you can create. *Type: int*
    interdc_gateway = resource2.Body('interdc_gateway', type=int)
    #: The maximum amount of internet gateway you can create. *Type: int*
    internet_gateway = resource2.Body('internet_gateway', type=int)
    #: The maximum amount of networks you can create. *Type: int*
    network = resource2.Body('network', type=int)
    #: The maximum amount of load balancer you can create. *Type: int*
    load_balancer = resource2.Body('load_balancer', type=int)
    #: The maximum amount of ports you can create. *Type: int*
    port = resource2.Body('port', type=int)
    #: The ID of the project these quota values are for.
    project_id = resource2.Body('tenant_id')
    #: The maximum amount of subnets you can create. *Type: int*
    subnet = resource2.Body('subnet', type=int)
    #: The maximum amount of vpn gateway you can create. *Type: int*
    vpn_gateway = resource2.Body('vpn_gateway', type=int)
    #: The maximum amount of public ip you can create. *Type: int*
    public_ip = resource2.Body('public_ip', type=int)


class QuotaDefault(Quota):
    base_path = '/quotas/%(project)s/default'

    # capabilities
    allow_retrieve = True
    allow_update = False
    allow_delete = False
    allow_list = False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestRole(base.BaseFunctionalTest):

    def test_01_create_role(self):
        role = self.conn.sss.create_role(
            tenant_id="5de7c7c21bba4939b071b3879172213e",
            user_id="ecid1000030914",
        )
        assert isinstance(role.user_id, six.string_types)
        assert isinstance(role.tenant_id, six.string_types)
        assert isinstance(role.tenant_name, six.string_types)
        assert isinstance(role.region_id, six.string_types)
        assert isinstance(role.region_name, six.string_types)
        assert isinstance(role.role_name, six.string_types)

    def test_02_delete_role(self):
        self.conn.sss.delete_role("5de7c7c21bba4939b071b3879172213e",
                                  "ecid1000030914")




# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestVirtualStorage(base.BaseFunctionalTest):
    def test_01_list_virtual_storages(self):
        vs = list(self.conn.storage.storages(False))
        print vs

    @classmethod
    def test_02_create_virtual_storage(cls):
        vs = cls.conn.storage.create_storage(
            name= "test-vs01-for-heat",
            network_id= "9d28c903-2bba-4900-a719-ab230a32b10b",
            subnet_id= "01c7f86b-a653-40fb-ab9c-ea7b8bb21f69",
            volume_type_id= "6328d234-7939-4d61-9216-736de66d15f9",
            ip_addr_pool= {"start": "10.1.1.21",
                "end": "10.1.1.30"}

        )
        cls.id = vs.id
        assert isinstance(vs.status, six.string_types)
        assert isinstance(vs.id, six.string_types)
        assert isinstance(vs.volume_type_id, six.string_types)
        assert isinstance(vs.name, six.string_types)
        assert isinstance(vs.network_id, six.string_types)
        assert isinstance(vs.ip_addr_pool, dict)
        assert isinstance(vs.subnet_id, six.string_types)
        assert isinstance(vs.host_routes, list)

    def test_03_show_virtual_storage(self):
        vs = self.conn.storage.get_storage(
            self.id
        )
        print vs
        self.assertIsInstance(vs.subnet_id, six.string_types)
        self.assertIsInstance(vs.host_routes, list)
        self.assertIsInstance(vs.status, six.string_types)
        self.assertIsInstance(vs.id, six.string_types)
        self.assertIsInstance(vs.volume_type_id, six.string_types)
        self.assertIsInstance(vs.name, six.string_types)
        self.assertIsInstance(vs.network_id, six.string_types)
        self.assertIsInstance(vs.ip_addr_pool, dict)

    def test_04_update_virtual_storage(self):
        vs = self.conn.storage.update_storage(
            self.id,
            description="sdk test"
        )
        self.assertIsInstance(vs.subnet_id, six.string_types)
        self.assertIsInstance(vs.host_routes, list)
        self.assertIsInstance(vs.status, six.string_types)
        self.assertIsInstance(vs.id, six.string_types)
        self.assertIsInstance(vs.volume_type_id, six.string_types)
        self.assertIsInstance(vs.name, six.string_types)
        self.assertIsInstance(vs.network_id, six.string_types)
        self.assertIsInstance(vs.ip_addr_pool, dict)
        self.assertIsInstance(vs.description, six.string_types)
        print vs.description

    def test_05_delete_virtual_storage(self):
        vs = self.conn.storage.delete_storage(
            self.id
        )
        